# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import os
import time
import unittest
import uuid

# import base class for unit testing
from .base_unit_test import BaseUnitTest

# import the package
import epages_client

# import the RestClient class
from epages_client.client import RestClient

# import the Category create class from dataobjects
from epages_client.dataobjects.category_create import CategoryCreate

# import the Category update class from dataobjects
from epages_client.dataobjects.category_update import CategoryUpdate

# import the Category sequence update class from dataobjects
from epages_client.dataobjects.category_sequence_update import CategorySequenceUpdate

# import the Product create class from dataobjects
from epages_client.dataobjects.product_create import ProductCreate

# import the Product patch class from dataobjects
from epages_client.dataobjects.product_update import ProductUpdate

# import the Product image sequence update class from dataobjects
from epages_client.dataobjects.product_slideshow_sequence_update import ProductSlideshowSequenceUpdate

from random import shuffle

skip_test = unittest.skipUnless(
    os.environ.get('EPAGES_RUN_ALL_TESTS', False), 'Skipping test.'
)


class TestProductMethods(BaseUnitTest):
    '''A class for testing product related methods on RestClient class'''

    # Get the directory where this file is located
    dir_path = os.path.dirname(os.path.realpath(__file__))

    # Set the resources directory where id files and test image are located
    resources = os.path.join(dir_path, "resources")

    # If the resources directory doesn't exist, raise an error
    if not os.path.exists(resources):
        raise OSError("Resources directory not found.")

    # Add category.txt to the resources path
    category_file = os.path.join(resources, "category.txt")

    # Add product.txt to the resources path
    product_file = os.path.join(resources, "product.txt")

    # The paths for the test images
    images = [
        os.path.join(resources, "test.jpg"),
        os.path.join(resources, "test2.jpg"),
        os.path.join(resources, "test3.jpg")
    ]

    # Loop images and check that every one of them is found
    for image in images:
        if not os.path.isfile(image):
            raise OSError("Image file '" + image + "' not found.")

    def get_category_ids(self):
        '''A function to get category ids from a text file.'''

        categories = []

        # Check if the category file is found and get the categories
        try:
            fh = open(self.category_file, "r")

            # Get lines to list
            lines = fh.read().splitlines()

            # Loop lines
            for line in lines:
                categories.append(line)

            fh.close()
        except FileNotFoundError:
            raise FileNotFoundError("Category file not found.")

        return categories

    def get_product_id(self):
        '''A function to get product id from a text file.'''

        product_id = ""

        try:
            fh = open(self.product_file, "r")
            product_id = fh.read()
            fh.close()
        except FileNotFoundError:
            raise FileNotFoundError("Product file not found.")

        return product_id

    def setUp(self):

        self.client = RestClient(
            os.environ["EPAGES_API_URL"], os.environ["EPAGES_API_TOKEN"])

        self.params = {
            "query": {},
            "param1": "",
            "param2": ""
        }

    def test_001_add_category_invalid_object(self):

        self.params["object"] = None

        with self.assertRaises(TypeError) as e:
            response = self.client.add_category(self.params)

    def test_002_add_category_no_root_category_id(self):

        category = CategoryCreate()
        self.params["object"] = category

        with self.assertRaises(ValueError) as e:
            response = self.client.add_category(self.params)

    def test_003_add_category(self):

        # Get categories
        categories = self.client.get_categories(self.params)

        # Get the id of the root category in the shop
        self.params["param1"] = categories[0]["categoryId"]

        category = CategoryCreate()
        self.params["object"] = category

        response = self.client.add_category(self.params)

        self.assertEqual(isinstance(response, dict), True)

        category_id = response["categoryId"]

        # Try to write the category id to the category file
        try:
            fh = open(self.category_file, "w")
            fh.write(category_id)
            fh.close()
        except IOError:
            print("Category file couldn't be created.")

    def test_004_add_category_2(self):

        # Wait for 1 second before creating a new category
        time.sleep(1)

        # Get categories
        categories = self.client.get_categories(self.params)

        # Get the id of the root category in the shop
        self.params["param1"] = categories[0]["categoryId"]

        category = CategoryCreate()
        self.params["object"] = category

        response = self.client.add_category(self.params)

        self.assertEqual(isinstance(response, dict), True)

        category_id = response["categoryId"]

        # Try to write the category id to the category file
        try:
            fh = open(self.category_file, "a")
            fh.write("\n" + category_id)
            fh.close()
        except IOError:
            print("Category file couldn't be opened.")

    def test_005_add_category_3(self):

        # Wait for 1 second before creating a new category
        time.sleep(1)

        # Get categories
        categories = self.client.get_categories(self.params)

        # Get the id of the root category in the shop
        self.params["param1"] = categories[0]["categoryId"]

        category = CategoryCreate()
        self.params["object"] = category

        response = self.client.add_category(self.params)

        self.assertEqual(isinstance(response, dict), True)

        category_id = response["categoryId"]

        # Try to write the category id to the category file
        try:
            fh = open(self.category_file, "a")
            fh.write("\n" + category_id)
            fh.close()
        except IOError:
            print("Category file couldn't be opened.")

    def test_006_add_product_no_object(self):

        with self.assertRaises(RuntimeError) as e:
            response = self.client.add_product(self.params)

    def test_007_add_product_invalid_object(self):

        self.params["object"] = None

        with self.assertRaises(TypeError) as e:
            response = self.client.add_product(self.params)

    def test_008_add_product_no_product_number(self):

        product = ProductCreate()

        self.params["object"] = product

        with self.assertRaises(ValueError) as e:
            response = self.client.add_product(self.params)

    def test_009_add_product(self):

        unique_id = str(uuid.uuid4())

        product = ProductCreate()
        product.productNumber = unique_id
        product.name = unique_id

        self.params["object"] = product

        response = self.client.add_product(self.params)

        self.assertEqual(isinstance(response, dict), True)

        product_id = response["productId"]

        # Try to write the product id to the product file
        try:
            fh = open(self.product_file, "w")
            fh.write(product_id)
            fh.close()
        except IOError:
            print("Product file couldn't be created.")

    def test_010_get_shop_info(self):

        shop_info = self.client.get_shop_info(self.params)

        self.assertEqual(isinstance(shop_info, dict), True)

    def test_011_get_categories(self):

        categories = self.client.get_categories(self.params)

        self.assertEqual(isinstance(categories, list), True)

    def test_012_get_category_no_id(self):

        with self.assertRaises(ValueError) as e:
            category = self.client.get_category(self.params)

    def test_013_get_category_false_id(self):

        self.params["param1"] = str(uuid.uuid4())

        with self.assertRaises(RuntimeError) as e:
            category = self.client.get_category(self.params)

    def test_014_get_category_correct_id(self):

        # Get all categories
        categories = self.client.get_categories(self.params)

        # Set category id to be the id of the first category
        self.params["param1"] = categories[0]["categoryId"]

        category = self.client.get_category(self.params)

        self.assertEqual(isinstance(category, dict), True)

    def test_015_get_currencies(self):

        currencies = self.client.get_currencies(self.params)

        self.assertEqual(isinstance(currencies, dict), True)

    def test_016_get_locales(self):

        locales = self.client.get_locales(self.params)

        self.assertEqual(isinstance(locales, dict), True)

    def test_017_get_products(self):

        # Set currency just to test currency setter
        self.client.currency = "GBP"
        
        products = self.client.get_products(self.params)

        self.assertEqual(isinstance(products, dict), True)

    def test_018_get_product_no_id(self):

        with self.assertRaises(ValueError) as e:
            product = self.client.get_product(self.params)

    def test_019_get_product_false_id(self):

        self.params["param1"] = str(uuid.uuid4())

        with self.assertRaises(RuntimeError) as e:
            product = self.client.get_product(self.params)

    def test_020_get_product_correct_id(self):

        product_id = self.get_product_id()

        self.params["param1"] = product_id

        product = self.client.get_product(self.params)

        self.assertEqual(isinstance(product, dict), True)

    def test_021_get_product_variations_no_id(self):

        with self.assertRaises(ValueError) as e:
            variations = self.client.get_product_variations(self.params)

    def test_022_get_product_variations_false_id(self):

        self.params["param1"] = str(uuid.uuid4())

        with self.assertRaises(RuntimeError) as e:
            variations = self.client.get_product_variations(self.params)

    @skip_test
    def test_023_get_product_variations_correct_id(self):

        # Here the product id is hard-coded, because there
        # isn't a way to get a product with variations
        self.params["param1"] = "5A4B8CC7-B4F1-236C-5497-0A2810120FFD"

        variations = self.client.get_product_variations(self.params)

        self.assertEqual(isinstance(variations, dict), True)

    def test_024_upload_product_image_no_id(self):

        with self.assertRaises(ValueError) as e:
            response = self.client.upload_product_image(self.params)

    def test_025_upload_product_image_no_data(self):

        product_id = self.get_product_id()

        self.params["param1"] = product_id

        with self.assertRaises(RuntimeError) as e:
            response = self.client.upload_product_image(self.params)

    def test_026_upload_product_image_1(self):

        product_id = self.get_product_id()
        self.params["param1"] = product_id

        image = open(self.images[0], 'rb')
        self.params["data"] = {'image': image}

        response = self.client.upload_product_image(self.params)

        image.close()

        self.assertEqual(isinstance(response, dict), True)

    def test_027_upload_product_image_2(self):

        product_id = self.get_product_id()
        self.params["param1"] = product_id

        image = open(self.images[1], 'rb')
        self.params["data"] = {'image': image}

        response = self.client.upload_product_image(self.params)

        image.close()

        self.assertEqual(isinstance(response, dict), True)

    def test_028_upload_product_image_3(self):

        product_id = self.get_product_id()
        self.params["param1"] = product_id

        image = open(self.images[2], 'rb')
        self.params["data"] = {'image': image}

        response = self.client.upload_product_image(self.params)

        image.close()

        self.assertEqual(isinstance(response, dict), True)

    def test_029_get_product_images_no_id(self):

        with self.assertRaises(ValueError) as e:
            images = self.client.get_product_images(self.params)

    def test_030_get_product_images_false_id(self):

        self.params["param1"] = str(uuid.uuid4())

        with self.assertRaises(RuntimeError) as e:
            images = self.client.get_product_images(self.params)

    def test_031_get_product_images_correct_id(self):

        product_id = self.get_product_id()
        self.params["param1"] = product_id

        images = self.client.get_product_images(self.params)

        self.assertEqual(isinstance(images, dict), True)

    def test_032_get_product_image_names_no_id(self):

        with self.assertRaises(ValueError) as e:
            images = self.client.get_product_image_names(self.params)

    def test_033_get_product_image_names_false_id(self):

        self.params["param1"] = str(uuid.uuid4())

        with self.assertRaises(RuntimeError) as e:
            images = self.client.get_product_image_names(self.params)

    def test_034_get_product_image_names_correct_id(self):

        product_id = self.get_product_id()
        self.params["param1"] = product_id

        images = self.client.get_product_image_names(self.params)

        self.assertEqual(isinstance(images, list), True)

    def test_035_get_product_custom_attributes_no_id(self):

        with self.assertRaises(ValueError) as e:
            attributes = self.client.get_product_custom_attributes(self.params)

    def test_036_get_product_custom_attributes_false_id(self):

        self.params["param1"] = str(uuid.uuid4())

        with self.assertRaises(RuntimeError) as e:
            attributes = self.client.get_product_custom_attributes(self.params)

    @skip_test
    def test_037_get_product_custom_attributes_correct_id(self):

        # Get products
        products = self.client.get_products(self.params)

        # Set product id to be the id of the first product
        self.params["param1"] = products["items"][0]["productId"]

        attributes = self.client.get_product_custom_attributes(self.params)

        self.assertEqual(isinstance(attributes, dict), True)

    def test_038_get_product_lowest_price_no_id(self):

        with self.assertRaises(ValueError) as e:
            lowest_price = self.client.get_product_lowest_price(self.params)

    def test_039_get_product_lowest_price_false_id(self):

        self.params["param1"] = str(uuid.uuid4())

        with self.assertRaises(RuntimeError) as e:
            lowest_price = self.client.get_product_lowest_price(self.params)

    @skip_test
    def test_040_get_product_lowest_price_correct_id(self):

        # Here the product id is hard-coded, because there
        # isn't a way to get a product with variations
        self.params["param1"] = "5A4B8CC7-B4F1-236C-5497-0A2810120FFD"

        lowest_price = self.client.get_product_lowest_price(self.params)

        self.assertEqual(isinstance(lowest_price, dict), True)

    def test_041_search_products_no_query(self):

        with self.assertRaises(ValueError) as e:
            search_results = self.client.search_products(self.params)

    def test_042_search_products_empty_query(self):

        self.params["query"] = {"query": ""}

        with self.assertRaises(ValueError) as e:
            search_results = self.client.search_products(self.params)

    def test_043_search_products(self):

        self.params["query"] = {"query": "snap"}

        search_results = self.client.search_products(self.params)

        self.assertEqual(isinstance(search_results, dict), True)

    def test_044_get_shipping_methods(self):

        shipping_methods = self.client.get_shipping_methods(self.params)

        self.assertEqual(isinstance(shipping_methods, list), True)

    def test_045_get_shipping_method_no_id(self):

        with self.assertRaises(ValueError) as e:
            shipping_method = self.client.get_shipping_method(self.params)

    def test_046_get_shipping_method_false_id(self):

        self.params["param1"] = str(uuid.uuid4())

        with self.assertRaises(RuntimeError) as e:
            shipping_method = self.client.get_shipping_method(self.params)

    def test_047_get_shipping_method_correct_id(self):

        # Get all shipping methods
        shipping_methods = self.client.get_shipping_methods(self.params)

        # Set shipping method id to be the id of the first shipping method
        self.params["param1"] = shipping_methods[0]["shippingMethodId"]

        shipping_method = self.client.get_shipping_method(self.params)

        self.assertEqual(isinstance(shipping_method, dict), True)

    def test_048_get_tax_classes(self):

        tax_classes = self.client.get_tax_classes(self.params)

        self.assertEqual(isinstance(tax_classes, dict), True)

    def test_049_get_tax_class_no_id(self):

        with self.assertRaises(ValueError) as e:
            tax_class = self.client.get_tax_class(self.params)

    def test_050_get_tax_class_false_id(self):

        self.params["param1"] = str(uuid.uuid4())

        with self.assertRaises(RuntimeError) as e:
            tax_class = self.client.get_tax_class(self.params)

    def test_051_get_tax_class_correct_id(self):

        # Get all tax classes
        tax_classes = self.client.get_tax_classes(self.params)

        # Set tax class id to be the id of the first tax class
        self.params["param1"] = tax_classes["items"][0]["taxClassId"]

        tax_class = self.client.get_tax_class(self.params)

        self.assertEqual(isinstance(tax_class, dict), True)

    def test_052_get_tax_model(self):

        tax_model = self.client.get_tax_model(self.params)

        self.assertEqual(isinstance(tax_model, dict), True)

    def test_053_update_category_no_id(self):

        with self.assertRaises(ValueError) as e:
            response = self.client.update_category(self.params)

    def test_054_update_category_no_object(self):

        categories = self.get_category_ids()
        self.params["param1"] = categories[0]

        with self.assertRaises(RuntimeError) as e:
            response = self.client.update_category(self.params)

    def test_055_update_category_invalid_object(self):

        categories = self.get_category_ids()
        self.params["param1"] = categories[0]
        self.params["object"] = None

        with self.assertRaises(TypeError) as e:
            response = self.client.update_category(self.params)

    def test_056_update_category_empty_object(self):
        '''When updating a category, the object must have category id and
        alias set. The category id must be the same that is in the url for
        updating category.'''

        categories = self.get_category_ids()
        self.params["param1"] = categories[0]
        self.params["object"] = CategoryUpdate()

        with self.assertRaises(ValueError) as e:
            response = self.client.update_category(self.params)

    def test_057_update_category_object_without_alias(self):

        categories = self.get_category_ids()
        self.params["param1"] = categories[0]

        category = CategoryUpdate()
        category.categoryId = categories[0]

        self.params["object"] = category

        with self.assertRaises(ValueError) as e:
            response = self.client.update_category(self.params)

    def test_058_update_category_mismatching_ids(self):
        '''In this test, both required variables are set, but category id
        has wrong value.'''

        categories = self.get_category_ids()

        # Correct category id is set to the final API path
        self.params["param1"] = categories[0]

        category = CategoryUpdate()

        # False category id is set to the object, and it causes mismatching ids error
        category.categoryId = str(uuid.uuid4())
        category.alias = str(uuid.uuid4())

        self.params["object"] = category

        with self.assertRaises(RuntimeError) as e:
            response = self.client.update_category(self.params)

    def test_059_update_category(self):

        categories = self.get_category_ids()
        self.params["param1"] = categories[0]

        category = CategoryUpdate()
        category.categoryId = categories[0]
        category.alias = str(uuid.uuid4())

        self.params["object"] = category

        response = self.client.update_category(self.params)

        self.assertEqual(isinstance(response, dict), True)

    def test_060_get_subcategory_sequence_no_id(self):

        with self.assertRaises(ValueError) as e:
            sequence = self.client.get_subcategory_sequence(self.params)

    def test_061_get_subcategory_sequence_false_id(self):

        self.params["param1"] = str(uuid.uuid4())

        with self.assertRaises(RuntimeError) as e:
            sequence = self.client.get_subcategory_sequence(self.params)

    def test_062_get_subcategory_sequence_correct_id(self):

        # Get all categories
        categories = self.client.get_categories(self.params)

        # Set category id to be the id of the first category
        self.params["param1"] = categories[0]["categoryId"]

        sequence = self.client.get_subcategory_sequence(self.params)

        self.assertEqual(isinstance(sequence, list), True)

    def test_063_update_subcategory_sequence_no_id(self):

        with self.assertRaises(ValueError) as e:
            sequence = self.client.update_subcategory_sequence(self.params)

    def test_064_update_subcategory_sequence_false_id(self):

        self.params["param1"] = str(uuid.uuid4())

        with self.assertRaises(RuntimeError) as e:
            sequence = self.client.update_subcategory_sequence(self.params)

    def test_065_update_subcategory_sequence_no_list(self):

        # Get all categories
        categories = self.client.get_categories(self.params)

        # Set category id to be the id of the first category
        self.params["param1"] = categories[0]["categoryId"]

        with self.assertRaises(RuntimeError) as e:
            response = self.client.update_subcategory_sequence(self.params)

    def test_066_update_subcategory_sequence_false_list(self):

        # Get all categories
        categories = self.client.get_categories(self.params)

        # Set category id to be the id of the first category
        self.params["param1"] = categories[0]["categoryId"]

        # Get the current subcategory sequence
        sequence = self.client.get_subcategory_sequence(self.params)

        # Add random item to the list to make sure this test fails
        sequence.append('testing')

        with self.assertRaises(RuntimeError) as e:
            response = self.client.update_subcategory_sequence(self.params)

    def test_067_update_subcategory_sequence_correct_id_and_list(self):

        # Get all categories
        categories = self.client.get_categories(self.params)

        # Set category id to be the id of the first category
        self.params["param1"] = categories[0]["categoryId"]

        # Get the category sequence
        sequence = self.client.get_subcategory_sequence(self.params)

        # Shuffle the sequence of categories
        shuffle(sequence)

        # Get the instance of category sequence update
        csu = CategorySequenceUpdate()

        # Loop shuffled categories and add them to the object
        for category in sequence:
            csu.add(category)

        # Add the object to the params
        self.params["object"] = csu

        response = self.client.update_subcategory_sequence(self.params)

        self.assertEqual(isinstance(response, list), True)

    def test_068_update_product_no_id(self):

        with self.assertRaises(ValueError) as e:
            response = self.client.update_product(self.params)

    def test_069_update_product_no_object(self):

        product_id = self.get_product_id()
        self.params["param1"] = product_id

        with self.assertRaises(RuntimeError) as e:
            response = self.client.update_product(self.params)

    def test_070_update_product_invalid_object(self):

        product_id = self.get_product_id()
        self.params["param1"] = product_id

        self.params["object"] = None

        with self.assertRaises(TypeError) as e:
            response = self.client.update_product(self.params)

    def test_071_update_product(self):

        product_id = self.get_product_id()
        self.params["param1"] = product_id

        product = ProductUpdate()
        product.price = 29.90
        product.stocklevel = 5

        self.params["object"] = product

        response = self.client.update_product(self.params)

        self.assertEqual(isinstance(response, dict), True)

    def test_072_update_product_image_sequence_no_id(self):

        with self.assertRaises(ValueError) as e:
            sequence = self.client.update_product_image_sequence(self.params)

    def test_073_update_product_image_sequence_false_id(self):

        self.params["param1"] = str(uuid.uuid4())

        with self.assertRaises(RuntimeError) as e:
            sequence = self.client.update_product_image_sequence(self.params)

    def test_074_update_product_image_sequence_no_list(self):

        product_id = self.get_product_id()
        self.params["param1"] = product_id

        with self.assertRaises(RuntimeError) as e:
            response = self.client.update_product_image_sequence(self.params)

    def test_075_update_product_image_sequence_false_list(self):

        product_id = self.get_product_id()
        self.params["param1"] = product_id

        # Get the current subcategory sequence
        sequence = self.client.get_product_image_names(self.params)

        # Add random item to the list to make sure this test fails
        sequence.append('testing')

        with self.assertRaises(RuntimeError) as e:
            response = self.client.update_product_image_sequence(self.params)

    def test_076_update_product_image_sequence_correct_id_and_list(self):

        product_id = self.get_product_id()
        self.params["param1"] = product_id

        # Get the image sequence
        sequence = self.client.get_product_image_names(self.params)

        # Shuffle the sequence of images
        shuffle(sequence)

        # Get the instance of product slideshow sequence update
        pssu = ProductSlideshowSequenceUpdate()

        # Loop shuffled images and add them to the object
        for image in sequence:
            pssu.add(image)

        # Add the object to the params
        self.params["object"] = pssu

        response = self.client.update_product_image_sequence(self.params)

        self.assertEqual(isinstance(response, list), True)

    def test_077_delete_product_image_no_id(self):

        with self.assertRaises(ValueError) as e:
            response = self.client.delete_product_image(self.params)

    def test_078_delete_product_image_false_id(self):

        self.params["param1"] = str(uuid.uuid4())
        self.params["param2"] = os.path.basename(self.images[0])

        with self.assertRaises(RuntimeError) as e:
            response = self.client.delete_product_image(self.params)

    def test_079_delete_product_image_no_name(self):

        product_id = self.get_product_id()
        self.params["param1"] = product_id

        with self.assertRaises(ValueError) as e:
            response = self.client.delete_product_image(self.params)

    def test_080_delete_product_image_false_name(self):

        product_id = self.get_product_id()
        self.params["param1"] = product_id
        self.params["param2"] = str(uuid.uuid4())

        with self.assertRaises(RuntimeError) as e:
            response = self.client.delete_product_image(self.params)

    def test_081_delete_product_image_1(self):

        product_id = self.get_product_id()
        self.params["param1"] = product_id
        self.params["param2"] = os.path.basename(self.images[0])

        response = self.client.delete_product_image(self.params)

        self.assertEqual(isinstance(response, dict), True)

    def test_082_delete_product_image_2(self):

        product_id = self.get_product_id()
        self.params["param1"] = product_id
        self.params["param2"] = os.path.basename(self.images[1])

        response = self.client.delete_product_image(self.params)

        self.assertEqual(isinstance(response, dict), True)

    def test_083_delete_product_image_3(self):

        product_id = self.get_product_id()
        self.params["param1"] = product_id
        self.params["param2"] = os.path.basename(self.images[2])

        response = self.client.delete_product_image(self.params)

        self.assertEqual(isinstance(response, dict), True)

    def test_084_get_updated_products_no_product_property(self):

        with self.assertRaises(ValueError) as e:
            response = self.client.get_updated_products(self.params)

    def test_085_get_updated_products_stocklevel(self):

        # Get products, whose stocklevel has been lately changed
        self.params["param1"] = "stocklevel"

        response = self.client.get_updated_products(self.params)

        self.assertEqual(isinstance(response, dict), True)

    def test_086_connect_category_and_product_no_data(self):

        with self.assertRaises(RuntimeError) as e:
            response = self.client.connect_category_and_product(self.params)

    def test_087_connect_category_and_product_false_data(self):

        self.params["data"] = {'categoryId': str(
            uuid.uuid4()), 'productId': str(uuid.uuid4())}

        with self.assertRaises(RuntimeError) as e:
            response = self.client.connect_category_and_product(self.params)

    def test_088_connect_category_and_product(self):

        categories = self.get_category_ids()
        product_id = self.get_product_id()

        self.params["data"] = {
            'categoryId': categories[0], 'productId': product_id}

        response = self.client.connect_category_and_product(self.params)

        self.assertEqual(isinstance(response, dict), True)

    def test_089_disconnect_product_and_category_no_data(self):

        with self.assertRaises(ValueError) as e:
            response = self.client.disconnect_product_and_category(self.params)

    def test_090_disconnect_product_and_category_false_data(self):

        self.params["query"]["categoryId"] = str(uuid.uuid4())
        self.params["query"]["productId"] = str(uuid.uuid4())

        with self.assertRaises(RuntimeError) as e:
            response = self.client.disconnect_product_and_category(self.params)

    def test_091_disconnect_product_and_category(self):

        categories = self.get_category_ids()
        product_id = self.get_product_id()

        self.params["query"]["categoryId"] = categories[0]
        self.params["query"]["productId"] = product_id

        response = self.client.disconnect_product_and_category(self.params)

        self.assertEqual(isinstance(response, dict), True)

    def test_092_get_watched_products(self):

        watched_products = self.client.get_watched_products(self.params)

        self.assertEqual(isinstance(watched_products, dict), True)

    def test_093_delete_product_no_id(self):

        with self.assertRaises(ValueError) as e:
            response = self.client.delete_product(self.params)

    def test_094_delete_product_false_id(self):

        self.params["param1"] = str(uuid.uuid4())

        with self.assertRaises(RuntimeError) as e:
            response = self.client.delete_product(self.params)

    def test_095_delete_product_correct_id(self):

        product_id = self.get_product_id()
        self.params["param1"] = product_id

        response = self.client.delete_product(self.params)

        self.assertEqual(isinstance(response, dict), True)

        # Remove the product file containing the product id/name
        os.remove(self.product_file)

    def test_096_delete_category_no_id(self):

        with self.assertRaises(ValueError) as e:
            response = self.client.delete_category(self.params)

    def test_097_delete_category_false_id(self):

        self.params["param1"] = str(uuid.uuid4())

        with self.assertRaises(RuntimeError) as e:
            response = self.client.delete_category(self.params)

    def test_098_delete_category(self):

        categories = self.get_category_ids()
        self.params["param1"] = categories[0]

        response = self.client.delete_category(self.params)

        self.assertEqual(isinstance(response, dict), True)

    def test_099_delete_category(self):

        # Wait for 1 second before deleting a category
        time.sleep(1)

        categories = self.get_category_ids()
        self.params["param1"] = categories[1]

        response = self.client.delete_category(self.params)

        self.assertEqual(isinstance(response, dict), True)

    def test_100_delete_category(self):

        # Wait for 1 second before deleting a category
        time.sleep(1)

        categories = self.get_category_ids()
        self.params["param1"] = categories[2]

        response = self.client.delete_category(self.params)

        self.assertEqual(isinstance(response, dict), True)

        # Remove the category file containing the category ids
        os.remove(self.category_file)

    def tearDown(self):
        pass



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from .data_object import DataObject
from .address import Address


class CustomerCreate(DataObject):
    ''''Data object for creating a new customer to ePages webshop'''

    def __init__(self):
        # string The number by which the merchant tracks the customer. If provided, the customer number must be unique. If not provided, the customer number will be generated automatically.
        self._customerNumber = None
        # string Internal note for the customer done by the merchant.
        self._internalNote = None
        # address The billing address of the customer (mandatory).
        self.billingAddress = Address()

    @property
    def customerNumber(self):
        return self._customerNumber

    @customerNumber.setter
    def customerNumber(self, value):
        self._customerNumber = self._check_str(
            value, "CustomerNumber has to be a str.")

    @property
    def internalNote(self):
        return self._internalNote

    @internalNote.setter
    def internalNote(self, value):
        self._internalNote = self._check_str(
            value, "InternalNote has to be a str.")

    def is_valid(self):
        # Get values for billing address
        billing_address_values = self.billingAddress.__dict__

        # Loop values and check, if there's even one set
        for key in billing_address_values:
            if billing_address_values[key] != None:
                return True

        return False



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from .data_object import DataObject


class ScriptTagCreate(DataObject):
    '''Data object for creating a new script tag to ePages webshop'''

    def __init__(self, scriptUrl=None):

        # string The URL of the script.
        self._scriptUrl = None if scriptUrl == None else self._check_url(
            scriptUrl, "ScriptUrl has to be a str.")

    @property
    def scriptUrl(self):
        return self._scriptUrl

    @scriptUrl.setter
    def scriptUrl(self, value):
        self._scriptUrl = self._check_url(value, "ScriptUrl has to be a str.")

    def is_valid(self):

        # Script url must be set always
        if not self._scriptUrl:
            return False
        else:
            return True



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from .data_object import DataObject


class Address(DataObject):
    '''Data object for creating an address object to ePages webshop api'''

    def __init__(self):
        self._company = None  # string The customer’s company.
        # string The customer’s salutation, e.g. Mr or Mrs.
        self._salutation = None
        # string The customer’s academic title, e.g. professor or PhD.
        self._title = None
        # string The customer’s first name.
        self._firstName = None
        # string The customer’s last name.
        self._lastName = None
        # string The street name of the customer’s address.
        self._street = None
        # string An additional field for the street address.
        self._streetDetails = None
        # string The zip or postal code of the address.
        self._zipCode = None
        # string The name of the city.
        self._city = None
        # string The name of the state.
        self._state = None
        # string The name of the country.
        self._country = None
        # string The Id of the VAT.
        self._vatId = None
        # string The customer’s date of birth.
        self._birthday = None
        # string The customer’s primary email address. This email address will be used by the merchant if the customer does not have a user account.
        self._emailAddress = None
        # string Further address details of the customer.
        self._addressExtension = None
        # string The name of the bank account holder.
        self._bankAccountHolder = None
        # string The bank account number.
        self._bankAccountNumber = None
        # string The name of the bank that holds the bank account.
        self._bankName = None
        # string The bank identifier code.
        self._bankSortCode = None
        # string The customer’s business email address.
        self._businessEmailAddress = None
        # string The customer’s business phone number.
        self._businessPhoneNumber = None
        # string The department of the company associated with the customer’s address.
        self._department = None
        # string Specifies the name to display for the customer name.
        self._displayName = None
        # string The door code associated with the customer’s address.
        self._doorCode = None
        # string The customer’s fax number.
        self._faxNumber = None
        # string The customer’s fiscal code.
        self._fiscalCode = None
        # string The customer’s gender. Can be either MALE or FEMALE.
        self._gender = None
        # string The customer’s job title.
        self._jobTitle = None
        # string The customer’s middle name.
        self._middleName = None
        # string The customer’s mobile phone number.
        self._mobilePhoneNumber = None
        # string The customer’s landline number.
        self._phoneNumber = None
        # string The customer’s private email address.
        self._privateEmailAddress = None
        # string The customer’s private phone number
        self._privatePhoneNumber = None
        # string The customer’s website URL.
        self._websiteUrl = None

    @property
    def company(self):
        return self._company

    @company.setter
    def company(self, value):
        self._company = self._check_str(value, "Company has to be a str.")

    @property
    def salutation(self):
        return self._salutation

    @salutation.setter
    def salutation(self, value):
        self._salutation = self._check_str(value, "Salutation has to be a str.")

    @property
    def title(self):
        return self._title

    @title.setter
    def title(self, value):
        self._title = self._check_str(value, "Title has to be a str.")

    @property
    def firstName(self):
        return self._firstName

    @firstName.setter
    def firstName(self, value):
        self._firstName = self._check_str(value, "FirstName has to be a str.")

    @property
    def lastName(self):
        return self._lastName

    @lastName.setter
    def lastName(self, value):
        self._lastName = self._check_str(value, "LastName has to be a str.")

    @property
    def street(self):
        return self._street

    @street.setter
    def street(self, value):
        self._street = self._check_str(value, "Street has to be a str.")

    @property
    def streetDetails(self):
        return self._streetDetails

    @streetDetails.setter
    def streetDetails(self, value):
        self._streetDetails = self._check_str(
            value, "StreetDetails has to be a str.")

    @property
    def zipCode(self):
        return self._zipCode

    @zipCode.setter
    def zipCode(self, value):
        self._zipCode = self._check_str(value, "ZipCode has to be a str.")

    @property
    def city(self):
        return self._city

    @city.setter
    def city(self, value):
        self._city = self._check_str(value, "City has to be a str.")

    @property
    def state(self):
        return self._state

    @state.setter
    def state(self, value):
        self._state = self._check_str(value, "State has to be a str.")

    @property
    def country(self):
        return self._country

    @country.setter
    def country(self, value):
        self._country = self._check_str(value, "Country has to be a str.")

    @property
    def vatId(self):
        return self._vatId

    @vatId.setter
    def vatId(self, value):
        self._vatId = self._check_str(value, "VatId has to be a str.")

    @property
    def birthday(self):
        return self._birthday

    @birthday.setter
    def birthday(self, value):
        self._birthday = self._check_str(value, "Birthday has to be a str.")

    @property
    def emailAddress(self):
        return self._emailAddress

    @emailAddress.setter
    def emailAddress(self, value):
        self._emailAddress = self._check_email(
            value, "EmailAddress has to be a valid email address.")

    @property
    def addressExtension(self):
        return self._addressExtension

    @addressExtension.setter
    def addressExtension(self, value):
        self._addressExtension = self._check_str(
            value, "AddressExtension has to be a str.")

    @property
    def bankAccountHolder(self):
        return self._bankAccountHolder

    @bankAccountHolder.setter
    def bankAccountHolder(self, value):
        self._bankAccountHolder = self._check_str(
            value, "BankAccountHolder has to be a str.")

    @property
    def bankAccountNumber(self):
        return self._bankAccountNumber

    @bankAccountNumber.setter
    def bankAccountNumber(self, value):
        self._bankAccountNumber = self._check_str(
            value, "BankAccountNumber has to be a str.")

    @property
    def bankName(self):
        return self._bankName

    @bankName.setter
    def bankName(self, value):
        self._bankName = self._check_str(value, "BankName has to be a str.")

    @property
    def bankSortCode(self):
        return self._bankSortCode

    @bankSortCode.setter
    def bankSortCode(self, value):
        self._bankSortCode = self._check_str(
            value, "BankSortCode has to be a str.")

    @property
    def businessEmailAddress(self):
        return self._businessEmailAddress

    @businessEmailAddress.setter
    def businessEmailAddress(self, value):
        self._businessEmailAddress = self._check_email(
            value, "BusinessEmailAddress has to be a valid email address.")

    @property
    def businessPhoneNumber(self):
        return self._businessPhoneNumber

    @businessPhoneNumber.setter
    def businessPhoneNumber(self, value):
        self._businessPhoneNumber = self._check_str(
            value, "BusinessPhoneNumber has to be a str.")

    @property
    def department(self):
        return self._department

    @department.setter
    def department(self, value):
        self._department = self._check_str(value, "Department has to be a str.")

    @property
    def displayName(self):
        return self._displayName

    @displayName.setter
    def displayName(self, value):
        self._displayName = self._check_str(
            value, "DisplayName has to be a str.")

    @property
    def doorCode(self):
        return self._doorCode

    @doorCode.setter
    def doorCode(self, value):
        self._doorCode = self._check_str(value, "DoorCode has to be a str.")

    @property
    def faxNumber(self):
        return self._faxNumber

    @faxNumber.setter
    def faxNumber(self, value):
        self._faxNumber = self._check_str(value, "FaxNumber has to be a str.")

    @property
    def fiscalCode(self):
        return self._fiscalCode

    @fiscalCode.setter
    def fiscalCode(self, value):
        self._fiscalCode = self._check_str(value, "FiscalCode has to be a str.")

    @property
    def gender(self):
        return self._gender

    @gender.setter
    def gender(self, value):
        if value in ('MALE', 'FEMALE'):
            self._gender = value
        else:
            raise ValueError(
                "The customer’s gender can be either MALE or FEMALE (str).")

    @property
    def jobTitle(self):
        return self._jobTitle

    @jobTitle.setter
    def jobTitle(self, value):
        self._jobTitle = self._check_str(value, "JobTitle has to be a str.")

    @property
    def middleName(self):
        return self._middleName

    @middleName.setter
    def middleName(self, value):
        self._middleName = self._check_str(value, "MiddleName has to be a str.")

    @property
    def mobilePhoneNumber(self):
        return self._mobilePhoneNumber

    @mobilePhoneNumber.setter
    def mobilePhoneNumber(self, value):
        self._mobilePhoneNumber = self._check_str(
            value, "MobilePhoneNumber has to be a str.")

    @property
    def phoneNumber(self):
        return self._phoneNumber

    @phoneNumber.setter
    def phoneNumber(self, value):
        self._phoneNumber = self._check_str(
            value, "PhoneNumber has to be a str.")

    @property
    def privateEmailAddress(self):
        return self._privateEmailAddress

    @privateEmailAddress.setter
    def privateEmailAddress(self, value):
        self._privateEmailAddress = self._check_email(
            value, "PrivateEmailAddress has to be a valid email address.")

    @property
    def privatePhoneNumber(self):
        return self._privatePhoneNumber

    @privatePhoneNumber.setter
    def privatePhoneNumber(self, value):
        self._privatePhoneNumber = self._check_str(
            value, "PrivatePhoneNumber has to be a str.")

    @property
    def websiteUrl(self):
        return self._websiteUrl

    @websiteUrl.setter
    def websiteUrl(self, value):
        self._websiteUrl = self._check_url(
            value, "WebsiteUrl has to be a valid url address.")

    def is_valid(self):
        return True



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from six import string_types
from .remove_value import RemoveValue
from .enum_fetch_operator import FetchOperator
import validators


class DataObject(object):
    '''Base class for all data_object classes'''

    def get_dict(self):
        '''
        Go over all values of object's _-named variables (and not None) and creates dictionary.
        The keys of the dictionary are variable names without "_"-mark.

        :return: the values of a dataobject for HttpHandler
        :rtype: dict
        '''
        self_values = self.__dict__
        data_dict = {}

        for key in self_values:
            if key[0] == '_' and self_values[key] != None:
                # Removes "_" from a variable name
                data_dict[key[1:]] = self_values[key]
            elif type(self_values[key]) in (RemoveValue,):
                data_dict[key] = self_values[key]
            elif isinstance(self_values[key], DataObject):
                obj_contents = self_values[key].get_dict()

                if obj_contents:  # pass empty lists and dictonaries
                    data_dict[key] = obj_contents

        return data_dict

    def get_list_of_json_patches(self, legal_operations):
        '''Go over all the variables of the data_object and return it like json_patch commands

        :param dict legal_operations: Key values are attribute paths and values are tuple of allowed FetchOperators.
        :return: Patch commands for API. A list of jsonPatches
        :return: list
        '''
        list_of_json_patches = []
        for key, value in self.get_dict().items():
            if value != {}:  # pass empty dict
                json_patch = {}

                # Replace underscores on key with a slash
                key = key.replace("_", "/")

                path = "/" + key
                json_patch['path'] = path
                if isinstance(value, RemoveValue):
                    if FetchOperator.REMOVE in legal_operations[path]:
                        json_patch['op'] = "remove"
                    else:
                        raise ValueError(path + " cannot remove")
                else:
                    if FetchOperator.ADD in legal_operations[path]:
                        json_patch['op'] = "add"
                        json_patch['value'] = value
                    elif FetchOperator.REPLACE in legal_operations[path]:
                        json_patch['op'] = "replace"
                        json_patch['value'] = value
                    else:
                        raise ValueError(
                            "There is not a valid fetch operator for " + path)
                list_of_json_patches.append(json_patch)
        return list_of_json_patches

    def _check_email(self, value, error_msg=None, allow_remove_value=False):
        '''
        Checks if the value is a valid email address.

        :param value: The value for check.
        :param str error_msg: The error message in ValueError case. Overwrites a default error message.
        :param bool allow_remove_value: Can the value be a RemoveValue?
        :return: The given value
        :rtype: str or RemoveValue.
        :raises ValueError: if the value is not a valid email address (or the value is a RemoveValue and allow_remove_value is True).
        '''

        if validators.email(value) or (allow_remove_value and isinstance(value, RemoveValue)):
            return value
        else:
            error_msg = error_msg or "The given value is not a valid email address."
            raise ValueError(error_msg)

    def _check_url(self, value, error_msg=None, allow_remove_value=False):
        '''
        Checks if the value is a valid url address.

        :param value: The value for check.
        :param str error_msg: The error message in ValueError case. Overwrites a default error message.
        :param bool allow_remove_value: Can the value be a RemoveValue?
        :return: The given value
        :rtype: str or RemoveValue.
        :raises ValueError: if the value is not a valid url address (or the value is a RemoveValue and allow_remove_value is True).
        '''

        if validators.url(value) or (allow_remove_value and isinstance(value, RemoveValue)):
            return value
        else:
            error_msg = error_msg or "The given value is not a valid url."
            raise ValueError(error_msg)

    def _check_str(self, value, error_msg=None, allow_remove_value=False):
        '''
        Checks if the value is str.

        :param value: The value for check.
        :param str error_msg: The error message in ValueError case. Overwrites a default error message.
        :param bool allow_remove_value: Can the value be a RemoveValue?
        :return: The given value
        :rtype: str or RemoveValue.
        :raises TypeError: if the value's type is not in str (or the value is a RemoveValue and allow_remove_value is True).
        '''
        error_msg = error_msg or 'The given value is not a str.'
        allow_types = string_types
        return self._check_type(value, error_msg, allow_types, allow_remove_value)

    def _check_bool(self, value, error_msg=None, allow_remove_value=False):
        '''
        Checks if the value is bool.

        :param value: The value for check.
        :param str error_msg: The error message in ValueError case. Overwrites a default error message.
        :param bool allow_remove_value: Can the value be a RemoveValue?
        :return: The given value
        :rtype: bool or RemoveValue.
        :raises TypeError: if the value's type is not in bool (or the value is a RemoveValue and allow_remove_value is True).
        '''
        error_msg = error_msg or 'The given value is not a bool.'
        allow_types = (bool,)
        return self._check_type(value, error_msg, allow_types, allow_remove_value)

    def _check_numeric(self, value, error_msg=None, allow_remove_value=False):
        '''
        Checks if the value is numeric (int or float)

        :param value: The value for check 
        :param str error_msg: The error message in ValueError case. Overwrites a default error message.
        :param bool allow_remove_value: Can the value be a RemoveValue?
        :return: The given value
        :rtype: int, float, or RemoveValue.
        :raises TypeError: if the value's type is not in int or float (or the value is a RemoveValue and allow_remove_value is True).
        '''

        error_msg = error_msg or 'The given value is not a numeric type.'
        allow_types = (int, float)
        return self._check_type(value, error_msg, allow_types, allow_remove_value)

    def _check_type(self, value, error_msg, allowed_types, allow_remove_value):
        '''
        Return the given value if its type is in the allowed_types.

        :param value: The value for check.
        :param str error_msg: The error message in ValueError case.
        :param tuple allowed_types: A tuple of classes.
        :paran bool allow_remove_value: Can the value be a RemoveValue object?
        :return: The given value if the value is one of allowed types or allow_remove_value is true and the value is an instance of RemoveValue.
        :raises TypeError: if the value's type is not in allowed_types (or the value is a RemoveValue and allow_remove_value is True).
        '''
        if isinstance(value, allowed_types) or (allow_remove_value and isinstance(value, RemoveValue)):
            return value
        else:
            raise TypeError(error_msg)

    def is_valid(self):
        '''Remember overwrite'''
        pass



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import os
import unittest

# import the RestClient class
from epages_client.client import RestClient

# Import the contact info class
from epages_client.dataobjects.contact_info import ContactInfo

# Import the content page class
from epages_client.dataobjects.content_page import ContentPage

# import base class for unit testing
from .base_unit_test import BaseUnitTest

skip_test = unittest.skipUnless(
    os.environ.get('EPAGES_RUN_ALL_TESTS', False), 'Skipping test.'
)


class TestLegalMethods(BaseUnitTest):
    '''A class for testing legal text related methods on RestClient class'''

    def setUp(self):

        self.client = RestClient(
            os.environ["EPAGES_API_URL"], os.environ["EPAGES_API_TOKEN"])

        self.params = {
            "query": {},
            "param1": "",
            "param2": ""
        }

    def test_001_get_legal_information(self):

        legal_information = self.client.get_legal_information(self.params)

        self.assertEqual(isinstance(legal_information, dict), True)

    def test_002_get_contact_information(self):

        contact_information = self.client.get_contact_information(self.params)

        self.assertEqual(isinstance(contact_information, dict), True)

    def test_003_get_privacy_policy(self):

        privacy_policy = self.client.get_privacy_policy(self.params)

        self.assertEqual(isinstance(privacy_policy, dict), True)

    def test_004_get_terms_and_conditions(self):

        terms_and_conditions = self.client.get_terms_and_conditions(
            self.params)

        self.assertEqual(isinstance(terms_and_conditions, dict), True)

    @skip_test
    def test_005_get_rights_of_withdrawal(self):

        rights_of_withdrawal = self.client.get_rights_of_withdrawal(
            self.params)

        self.assertEqual(isinstance(rights_of_withdrawal, dict), True)

    def test_006_get_shipping_information(self):

        shipping_information = self.client.get_shipping_information(
            self.params)

        self.assertEqual(isinstance(shipping_information, dict), True)

    def test_007_update_contact_information_no_locale(self):

        with self.assertRaises(ValueError) as e:
            response = self.client.update_contact_information(self.params)

    def test_008_update_contact_information_unsupported_locale(self):

        self.client.locale = "de_DE"

        with self.assertRaises(RuntimeError) as e:
            response = self.client.update_contact_information(self.params)

    def test_009_update_contact_information_invalid_object(self):

        self.client.locale = "fi_FI"
        self.params["object"] = None

        with self.assertRaises(TypeError) as e:
            response = self.client.update_category(self.params)

    def test_010_update_contact_information(self):

        self.client.locale = "en_GB"

        contact_info = ContactInfo()

        contact_info.name = "Contact information"
        contact_info.company = "Test shop"
        contact_info.address = "Finlaysoninkuja 19\n"
        contact_info.address += "33210 Tampere"

        self.params["object"] = contact_info

        response = self.client.update_contact_information(self.params)

        self.assertEqual(isinstance(response, dict), True)

    def test_011_update_privacy_policy_no_locale(self):

        with self.assertRaises(ValueError) as e:
            response = self.client.update_privacy_policy(self.params)

    def test_012_update_privacy_policy(self):

        self.client.locale = "en_GB"

        privacy_policy = self.client.get_privacy_policy(self.params)

        content_page = ContentPage()

        content_page.name = privacy_policy["name"]
        content_page.navigationCaption = privacy_policy["navigationCaption"]
        content_page.title = privacy_policy["title"]
        content_page.description = privacy_policy["description"]

        self.params["object"] = content_page

        response = self.client.update_privacy_policy(self.params)

        self.assertEqual(isinstance(response, dict), True)

    def test_013_update_terms_and_conditions_no_locale(self):

        with self.assertRaises(ValueError) as e:
            response = self.client.update_terms_and_conditions(self.params)

    def test_014_update_terms_and_conditions(self):

        self.client.locale = "en_GB"

        terms_and_conditions = self.client.get_terms_and_conditions(
            self.params)

        content_page = ContentPage()

        content_page.name = terms_and_conditions["name"]
        content_page.navigationCaption = terms_and_conditions["navigationCaption"]
        content_page.title = terms_and_conditions["title"]
        content_page.description = terms_and_conditions["description"]

        self.params["object"] = content_page

        response = self.client.update_terms_and_conditions(self.params)

        self.assertEqual(isinstance(response, dict), True)

    def test_015_update_rights_of_withdrawal_no_locale(self):

        with self.assertRaises(ValueError) as e:
            response = self.client.update_rights_of_withdrawal(self.params)

    @skip_test
    def test_016_update_rights_of_withdrawal(self):

        self.client.locale = "en_GB"

        content_page = ContentPage()

        content_page.name = "Rights of withdrawal"
        content_page.navigationCaption = "Rights of withdrawal"
        content_page.title = "Rights of withdrawal - Test shop"
        content_page.description = "There isn't any rights of withdrawal, deal with it :)"

        self.params["object"] = content_page

        response = self.client.update_rights_of_withdrawal(self.params)

        self.assertEqual(isinstance(response, dict), True)

    def test_017_update_shipping_information_no_locale(self):

        with self.assertRaises(ValueError) as e:
            response = self.client.update_shipping_information(self.params)

    def test_018_update_shipping_information(self):

        self.client.locale = "en_GB"

        content_page = ContentPage()

        content_page.name = "Shipping information"
        content_page.navigationCaption = "Shipping information"
        content_page.title = "Shipping information - Test shop"
        content_page.description = "Along the shipping information there should be also a boating information ;)"

        self.params["object"] = content_page

        response = self.client.update_shipping_information(self.params)

        self.assertEqual(isinstance(response, dict), True)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from .data_object import DataObject


class ProductLineItemCreate(DataObject):
    """Data object for ProductLineItemCreate"""

    def __init__(self):
        # string The unique identifier of the product.
        self._productId = None
        # number The product quantity of this line item displayed as a decimal number.
        self._quantity = None

    @property
    def productId(self):
        return self._productId

    @productId.setter
    def productId(self, value):
        self._productId = self._check_str(value, "ProductId has to be a str.")

    @property
    def quantity(self):
        return self._quantity

    @quantity.setter
    def quantity(self, value):
        self._quantity = self._check_numeric(
            value, "Quantity has to be a numeric type.")

    def is_valid(self):
        return True



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import unittest

# import the package
import epages_client

from epages_client.dataobjects.category_update import CategoryUpdate

# import base class for unit testing
from .base_unit_test import BaseUnitTest


class TestStringMethods(BaseUnitTest):
    def setUp(self):
        pass

    def test_0001_simple_correct_inputs(self):
        category = CategoryUpdate()
        # string The unique identifier of the category a product is assigned to.
        category.categoryId = '111-222-333-444'
        # string The name of the category.
        category.name = 'Cat'
        # string The unique identifier of the category.
        category.alias = 'pets_cats'
        # string The page title of this category.
        category.pageTitle = 'Cats'
        # string The description of the category.
        category.description = 'Products for cats'
        # string The name of the category page that appears in the navigation bar.
        category.navigationCaption = 'Navi Cat'
        # boolean Indicates if the category is displayed in the shop.
        category.visible = False

        right_answer = {
            'categoryId': '111-222-333-444',
            'name': 'Cat',
            'alias': 'pets_cats',
            'pageTitle': 'Cats',
            'description': 'Products for cats',
            'navigationCaption': 'Navi Cat',
            'visible': False
        }

        self.assertEqual(category.get_dict(), right_answer)

    def test_0002_incorrect_inputs(self):
        category = CategoryUpdate()
        with self.assertRaises(TypeError) as e:
            category.categoryId = 111-222-333-444


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import unittest

# import the package
import epages_client

from epages_client.dataobjects.content_page import ContentPage

# import base class for unit testing
from .base_unit_test import BaseUnitTest


class TestStringMethods(BaseUnitTest):
    def setUp(self):
        pass

    def test_0001_simple_correct_inputs(self):
        page = ContentPage()

        # string The name that appears on the page.
        page.name = "Cats"
        # string The name of the page, that appears on the browser tab.
        page.title = "Cats' toys"
        # string The name of the page, that appears in the navigation bar.
        page.navigationCaption = "Toys for cats"
        # string Additional short information that can be given to better explain what’s on the page.
        page.shortDescription = "short description"
        # string Information on the topic of the page.
        page.description = "long description"

        right_answer = {
            'name': 'Cats',
            'title': "Cats' toys",
            'navigationCaption': 'Toys for cats',
            'shortDescription': 'short description',
            'description': 'long description',
        }

        self.assertEqual(page.get_dict(), right_answer)

    def test_0002_incorrect_inputs(self):
        page = ContentPage()
        with self.assertRaises(TypeError) as e:
            page.name = 123


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from .customer_create import CustomerCreate
from .enum_fetch_operator import FetchOperator


class CustomerUpdate(CustomerCreate):
    '''Data object for updating a customer to ePages webshop'''

    def __init__(self):
        super(CustomerUpdate, self).__init__()
        self.legals = {
            '/customerNumber': (FetchOperator.ADD,),
            '/billingAddress': (FetchOperator.ADD, FetchOperator.REMOVE),
            '/internalNote': (FetchOperator.ADD, FetchOperator.REMOVE),
        }

    @property
    def internalNote(self):
        return self._internalNote

    @internalNote.setter
    def internalNote(self, value):
        self._internalNote = self._check_str(value, "InternalNote has to be a str or RemoveValue.", True)

    def get_patch(self):
        return self.get_list_of_json_patches(self.legals)

    def is_valid(self):
        return self.billingAddress.is_valid()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import unittest

# import the package
import epages_client

from epages_client.dataobjects.customer_create import CustomerCreate

# import base class for unit testing
from .base_unit_test import BaseUnitTest


class TestStringMethods(BaseUnitTest):
    def setUp(self):
        pass

    def test_0001_correct_inputs(self):

        james = CustomerCreate()
        james.customerNumber = '007'
        james.billingAddress.gender = 'MALE'
        james.billingAddress.emailAddress = 'james@bond.uk'

        self.assertEqual(james.get_dict(), {'customerNumber': '007', 'billingAddress': {
                         'emailAddress': 'james@bond.uk', 'gender': 'MALE'}})

        maria = CustomerCreate()
        maria.customerNumber = '23'
        self.assertEqual(maria.customerNumber, '23')
        maria.internalNote = "What a customer!"
        self.assertEqual(maria.internalNote, 'What a customer!')
        maria.billingAddress.doorCode = "Simsalabim"
        self.assertEqual(maria.is_valid(), True)

        self.assertEqual(maria.billingAddress.get_dict(),
                         {'doorCode': 'Simsalabim'})

    def test_0002_invalid_inputs(self):
        bad_lad = CustomerCreate()

        with self.assertRaises(ValueError) as e:
            bad_lad.billingAddress.emailAddress = 'my(a)emailaddress.com'

        with self.assertRaises(ValueError) as e:
            bad_lad.billingAddress.gender = 'MAN'


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import unittest
from pprint import pprint

# import the package
import epages_client

from epages_client.dataobjects.script_tag_create import ScriptTagCreate

# import base class for unit testing
from .base_unit_test import BaseUnitTest


class TestStringMethods(BaseUnitTest):
    def setUp(self):
        pass

    def test_0001_dont_set_url_init(self):
        url = ScriptTagCreate()
        self.assertEqual(url.scriptUrl, None)
        self.assertEqual(url.is_valid(), False)

    def test_0002_set_valid_url_init(self):
        url = ScriptTagCreate(
            "https://ajax.googleapis.com/ajax/libs/dojo/1.13.0/dojo/dojo.js")
        self.assertEqual(
            url.scriptUrl, "https://ajax.googleapis.com/ajax/libs/dojo/1.13.0/dojo/dojo.js")
        self.assertEqual(url.is_valid(), True)

    def test_0003_set_valid_url(self):
        url = ScriptTagCreate()
        url.scriptUrl = "https://ajax.googleapis.com/ajax/libs/jquery/3.3.1/jquery.min.js"
        self.assertEqual(
            url.scriptUrl, "https://ajax.googleapis.com/ajax/libs/jquery/3.3.1/jquery.min.js")
        self.assertEqual(url.is_valid(), True)

    def test_0004_set_invalid_url_init(self):
        url = ScriptTagCreate()
        with self.assertRaises(ValueError) as e:
            url = ScriptTagCreate(
                "://ajax.googleapis.com/ajax/libs/dojo/1.13.0/dojo/dojo.js")

    def test_0004_set_invalid_url(self):
        url = ScriptTagCreate()
        with self.assertRaises(ValueError) as e:
            url.scriptUrl = "://ajax.googleapis.com/ajax/libs/dojo/1.13.0/dojo/dojo.js"



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import unittest

# import the package
import epages_client

from epages_client.dataobjects.contact_info import ContactInfo

# import base class for unit testing
from .base_unit_test import BaseUnitTest


class TestStringMethods(BaseUnitTest):
    def setUp(self):
        pass

    def test_0001_simple_correct_inputs(self):
        c_info = ContactInfo()

        # string The name that appears on the contact information page, e.g. Contact Us.
        c_info.name = 'New cat products'
        # string The name of the contact information page, that appears on the browser tab.
        c_info.title = 'Cat products'
        # string The name of the contact information page that appears in the navigation bar.
        c_info.navigationCaption = 'navigationCaption'
        # string Additional short information that can be given to e.g. better explain what’s on the contact information page.
        c_info.shortDescription = 'short description'
        # string Additional information that can be added to the contact information page, e.g. tax identification number or bank account.
        c_info.description = 'Description long'
        # string The name of the shop.
        c_info.company = 'Company Ltd'
        # string The contact person for the shop, usually the shop owner.
        c_info.contactPerson = 'Öölö Köölö'
        # string The job title of the contact person.
        c_info.contactPersonJobTitle = 'Director'
        # string The postal address of the shop.
        c_info.address = 'Street'
        # string The phone number of the shop.
        c_info.phone = '020 7431912'
        # string The email address of the shop.
        c_info.email = 'email@address.invalid'

        right_answer = {
            'name': 'New cat products',
            'title': 'Cat products',
            'navigationCaption': 'navigationCaption',
            'shortDescription': 'short description',
            'description': 'Description long',
            'company': 'Company Ltd',
            'contactPerson': 'Öölö Köölö',
            'contactPersonJobTitle': 'Director',
            'address': 'Street',
            'phone': '020 7431912',
            'email': 'email@address.invalid'
        }

        self.assertEqual(c_info.get_dict(), right_answer)

    def test_0002_incorrect_inputs(self):
        c_info = ContactInfo()
        with self.assertRaises(ValueError) as e:
            c_info.email = "email(at)address.fi"


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

"""Tests for `epages_client.dataobjects.product` package."""
import unittest

# import the package
import epages_client

from epages_client.dataobjects.product_create import ProductCreate

# import base class for unit testing
from .base_unit_test import BaseUnitTest


class TestStringMethods(BaseUnitTest):
    def setUp(self):
        self.product = ProductCreate()

    def test_0001_correct_inputs(self):

        # product_number missing
        self.assertEqual(self.product.is_valid(), False)

        self.product.productNumber = "ZZZ-ZZZ-2"
        self.assertEqual(self.product.productNumber, "ZZZ-ZZZ-2")

        # product_number set
        self.assertEqual(self.product.is_valid(), True)

        self.product.name = "product 1"
        self.assertEqual(self.product.name, "product 1")

        self.product.shortDescription = "A nice product"
        self.assertEqual(self.product.shortDescription, "A nice product")

        self.product.description = "Very nice and cool product"
        self.assertEqual(self.product.description,
                         "Very nice and cool product")

        self.product.manufacturer = "Busines Oy"
        self.assertEqual(self.product.manufacturer, "Busines Oy")

        self.product.price = 2
        self.assertEqual(self.product.price, 2)

        self.product.price = 2.30
        self.assertEqual(self.product.price, 2.30)

        self.product.essentialFeatures = "Jump and speak"
        self.assertEqual(self.product.essentialFeatures, "Jump and speak")

        self.product.upc = '614141000036'
        self.assertEqual(self.product.upc, '614141000036')

        self.product.ean = '9501101530003'
        self.assertEqual(self.product.ean, '9501101530003')

        self.product.deliveryPeriod = '1 day'
        self.assertEqual(self.product.deliveryPeriod, '1 day')

        self.product.visible = True
        self.assertEqual(self.product.visible, True)

        self.product.taxClassId = '2'
        self.assertEqual(self.product.taxClassId, '2')

        self.product.stocklevel = 1000
        self.assertEqual(self.product.stocklevel, 1000)

        self.product.depositPrice = 0.49
        self.assertEqual(self.product.depositPrice, 0.49)

        self.product.manufacturerPrice = 5.99
        self.assertEqual(self.product.manufacturerPrice, 5.99)

        self.product.searchKeywords.add('cat')
        self.product.searchKeywords.add('doc')
        self.assertEqual(self.product.searchKeywords.get(), ['cat', 'doc'])

        self.assertEqual(self.product.get_dict(), {'taxClassId': '2', 'shortDescription': 'A nice product', 'price': 2.3, 'essentialFeatures': 'Jump and speak', 'visible': True, 'depositPrice': 0.49, 'stocklevel': 1000, 'manufacturer': 'Busines Oy', 'searchKeywords': [
                         'cat', 'doc'], 'manufacturerPrice': 5.99, 'productNumber': 'ZZZ-ZZZ-2', 'upc': '614141000036', 'ean': '9501101530003', 'name': 'product 1', 'deliveryPeriod': '1 day', 'description': 'Very nice and cool product'})

    def test_0002_invalid_inputs(self):
        product = ProductCreate()

        with self.assertRaises(TypeError) as e:
            product.price = '2.20 €'

        with self.assertRaises(TypeError) as e:
            product.visible = 'True'

        with self.assertRaises(TypeError) as e:
            product.stocklevel = '2000 pcs.'

        with self.assertRaises(TypeError) as e:
            product.depositPrice = 5, 59

        with self.assertRaises(TypeError) as e:
            product.manufacturerPrice = '1.85'

        self.assertEqual(product.get_dict(), {})
        self.assertEqual(product.is_valid(), False)


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import os
import time
import uuid
import time
from pprint import pprint

# import base class for unit testing
from .base_unit_test import BaseUnitTest

# import the package
import epages_client

# import the RestClient class
from epages_client.client import RestClient

# import Dataobjects
from epages_client.dataobjects.cart_create import CartCreate
from epages_client.dataobjects.product_line_item_create import ProductLineItemCreate
from epages_client.dataobjects.product_line_item_update import ProductLineItemUpdate
from epages_client.dataobjects.address import Address
from epages_client.dataobjects.order_update import OrderUpdate


class TestCartsOrdersAndOrdersMethods(BaseUnitTest):
    '''A class for testing order related methods on RestClient class'''

    # filename for saving cart credential
    cart_file = "cart_credential.csv"

    # filename for saving couponLineItemId
    coupon_line_file = "cart_coupon_line.txt"

    # filename for saving lineItemId
    product_line_file = "cart_product_line_item_id.txt"

    # filename for saving order id
    order_id_file = "order_id.txt"

    # Code for a coupon
    coupun_code = 'TEST-CODE-ABC123'

    def add_cart_credential(self, params):
        '''This function adds cart token and cart id to given params from csv file.'''

        content = self.get_resource(self.cart_file)

        ids = content.split(";")

        params.update({
            'param1': ids[0],
            'headers': {'X-ePages-Cart-Token': ids[1]}
        })

        return params

    def save_cart_credential(self, response):
        '''Function to save cart token and cart_id to csv-file'''

        content = "%s;%s" % (
            response['X-ePages-Cart-Token'], response['cartId'])

        self.save_resource(self.cart_file, content)

    def get_products(self):
        products = self.client.get_products({
            "query": {},
            "param1": "",
            "param2": ""
        })
        return products['items']

    def setUp(self):

        self.client = RestClient(
            os.environ["EPAGES_API_URL"], os.environ["EPAGES_API_TOKEN"])

        self.params = {
            "query": {},
            "param1": "",
            "param2": ""
        }

    def test_0001_create_empty_cart(self):
        # Create a cart without products

        # Create a new cart
        cart = CartCreate()
        self.params["object"] = cart
        response = self.client.add_cart(self.params)
        self.assertEqual(isinstance(response, dict), True)

        # Save cartId credentials to csv file
        self.save_cart_credential(response)

    def test_0002_get_cart(self):
        # Returns a specific cart from a shop

        # set credential of cart
        self.params = self.add_cart_credential(self.params)

        response = self.client.get_cart(self.params)
        self.assertEqual(isinstance(response, dict), True)

    def test_0003_add_product_line(self):
        # Creates a product line item in a cart.

        # set credential of cart
        self.params = self.add_cart_credential(self.params)

        # Get one product
        product_id = self.get_products()[0]['productId']

        # Crate product order line
        line = ProductLineItemCreate()
        line.productId = product_id
        line.quantity = 1

        self.params["object"] = line

        response = self.client.add_cart_line_item(self.params)
        self.assertEqual(isinstance(response, dict), True)

        # We'll need lineItemId for updating and removing a product from the cart
        self.save_resource(
            self.product_line_file, response['lineItemContainer']['productLineItems'][0]['lineItemId'].strip())

    def test_0004_update_product_line(self):
        # Update quantity of product line

        line = ProductLineItemUpdate()
        line.quantity = 5

        # set credential of cart
        self.params = self.add_cart_credential(self.params)
        self.params["param2"] = self.get_resource(self.product_line_file)
        self.params["object"] = line

        response = self.client.update_cart_line_item(self.params)
        self.assertEqual(isinstance(response, dict), True)

    def test_0005_add_coupon_to_cart(self):
        # Applies a coupon code on a cart

        # set credential of cart
        self.params = self.add_cart_credential(self.params)
        # x-www-form-urlencoded
        self.params["data"] = {'code': self.coupun_code}

        response = self.client.add_coupon(self.params)
        self.assertEqual(isinstance(response, dict), True)

        self.save_resource(
            self.coupon_line_file, response['lineItemContainer']['couponLineItem']['couponLineItemId'].strip())

    def test_0006_delete_coupon_from_cart(self):
        # Deletes a coupon from a cart

        # set credential of cart
        self.params = self.add_cart_credential(self.params)
        self.params["param2"] = self.get_resource(self.coupon_line_file)
        # x-www-form-urlencoded
        self.params["data"] = {'code': self.coupun_code}

        response = self.client.delete_coupon(self.params)
        self.assertEqual(isinstance(response, dict), True)

    def test_0007_delete_product_line(self):
        # Removes a product line item from a cart.

        # set credentials of the cart
        self.params = self.add_cart_credential(self.params)
        self.params["param2"] = self.get_resource(self.product_line_file)

        response = self.client.delete_cart_line_item(self.params)
        self.assertEqual(isinstance(response, dict), True)

    def test_0008_add_billing_address(self):
        # Modifies the billing address for a cart.

        billing_address = Address()
        billing_address.firstName = "Äijö"
        billing_address.lastName = "Äälinen"
        billing_address.street = "Pellavatehtaankatu 19"
        billing_address.zipCode = "33210"
        billing_address.city = "Tampere"
        billing_address.country = "FI"
        billing_address.emailAddress = "aijo.aalinen@vilkas.invalid"

        # set credentials of the cart
        self.params = self.add_cart_credential(self.params)
        self.params["object"] = billing_address

        response = self.client.update_billing_address(self.params)
        self.assertEqual(isinstance(response, dict), True)

    def test_0009_delete_billing_address(self):
        # set credentials of the cart
        self.params = self.add_cart_credential(self.params)

        response = self.client.delete_billing_address(self.params)
        self.assertEqual(isinstance(response, dict), True)

    def test_0010_add_shipping_address(self):
        # Modifies the shipping address for a cart.

        shipping_address = Address()
        shipping_address.firstName = "Лев"
        shipping_address.lastName = "Толстой"
        shipping_address.street = "Pellavatehtaankatu 19"
        shipping_address.zipCode = "33210"
        shipping_address.city = "Tampere"
        shipping_address.country = "FI"
        shipping_address.emailAddress = "leo.tolstoy@vilkas.invalid"
        shipping_address.gender = "MALE"
        shipping_address.jobTitle = "writer"

        # set credentials of the cart
        self.params = self.add_cart_credential(self.params)
        self.params["object"] = shipping_address

        response = self.client.update_shipping_address(self.params)
        self.assertEqual(isinstance(response, dict), True)

    def test_0011_shipping_shipping_address(self):
        # set credentials of the cart
        self.params = self.add_cart_credential(self.params)

        response = self.client.delete_shipping_address(self.params)
        self.assertEqual(isinstance(response, dict), True)

    def test_1001_add_order(self):
        # create a cart
        cart = CartCreate()
        cart.currency = 'EUR'
        cart.locale = 'fi_fi'
        cart.taxType = 'NET'

        products = self.get_products()

        # Add products to the cart
        item1 = ProductLineItemCreate()
        item1.productId = products[0]['productId']
        item1.quantity = 2
        cart.lineItems.add(item1)

        item2 = ProductLineItemCreate()
        item2.productId = products[1]['productId']
        item2.quantity = 4
        cart.lineItems.add(item2)

        time.sleep(1)

        # send to a shop
        self.params["object"] = cart
        response = self.client.add_cart(self.params)
        self.save_cart_credential(response)

        basic_params = self.add_cart_credential({})

        # Add required billing address
        buyer = Address()
        buyer.country = "FI"
        buyer.emailAddress = "yrjo.aaninen@vilkas.invalid"

        # params for update billing address
        params = {'object': buyer}
        params.update(basic_params)

        time.sleep(1)

        response = self.client.update_billing_address(params)
        self.assertEqual(isinstance(response, dict), True)

        time.sleep(1)

        response = self.client.add_order(basic_params)
        self.assertEqual(isinstance(response, dict), True)

        self.save_resource(self.order_id_file, response['orderId'])

    def test_1002_get_orders(self):
        # Get orders

        # find all finnish and euro orders
        self.params['query'] = {
            'currency': 'EUR',
            'locale': 'fi_FI'
        }

        response = self.client.get_orders(self.params)
        self.assertEqual(isinstance(response, dict), True)

    def test_1003_get_the_order(self):
        # Get the order created before

        # Get order id
        self.params["param1"] = self.get_resource(self.order_id_file)

        response = self.client.get_order(self.params)
        self.assertEqual(isinstance(response, dict), True)

    def test_1004_update_order(self):
        # update the order created before

        order = OrderUpdate()
        order.billingAddress.emailAddress = "my-new@email.address.invalid"
        order.billingAddress.firstName = "David"
        order.billingAddress.lastName = "Mattson"
        order.billingAddress.lastName = "MALE"

        # Get order id
        self.params["param1"] = self.get_resource(self.order_id_file)
        self.params["object"] = order

        response = self.client.update_order(self.params)
        self.assertEqual(isinstance(response, dict), True)

    def test_1005_get_order_documents(self):
        # Returns finalized invoice and credit note order documents by orderId.

        # Get order id
        self.params["param1"] = self.get_resource(self.order_id_file)

        response = self.client.get_order_documents(self.params)
        self.assertEqual(isinstance(response, dict), True)

    def test_2001_try_add_order_without_billing_address(self):
        # User forget update billing address

        # create a cart
        response = self.client.add_cart(self.params)
        self.save_cart_credential(response)

        basic_params = self.add_cart_credential({})

        # Billing address is missing!
        with self.assertRaises(RuntimeError) as e:
            response = self.client.add_order(basic_params)

    def test_2002_try_add_billing_address_without_params_object(self):
        # User forget add object to params

        # set credential of cart
        self.params = self.add_cart_credential(self.params)

        billing_address = Address()
        billing_address.firstName = "Test"
        billing_address.lastName = "Männen"
        billing_address.email = "test@mannen.com"
        billing_address.country = "FI"

        # billingAddress is missing!
        with self.assertRaises(RuntimeError) as e:
            response = self.client.update_shipping_address(self.params)

    def test_2003_try_add_billing_address_without_country(self):
        # Try add a billing address without country

        # set credential of cart
        self.params = self.add_cart_credential(self.params)

        billing_address = Address()
        billing_address.firstName = "Test"
        billing_address.lastName = "Männen"
        billing_address.email = "test@mannen.com"

        self.params['object'] = billing_address

        # Country not Valid (missing)
        with self.assertRaises(RuntimeError) as e:
            response = self.client.update_shipping_address(self.params)

    def test_2004_try_add_billing_address_without_email(self):
        # Try add a billing address without email

        # set credential of cart
        self.params = self.add_cart_credential(self.params)

        billing_address = Address()
        billing_address.firstName = "Test"
        billing_address.lastName = "Männen"
        billing_address.country = "GB"

        self.params['object'] = billing_address

        # Country not Valid (missing)
        with self.assertRaises(RuntimeError) as e:
            response = self.client.update_billing_address(self.params)

    def test_2005_add_order_twice(self):
        # try two times create order from a cart

        # Add correct billing address
        basic_params = self.add_cart_credential(self.params)
        billing_address = Address()
        billing_address.country = "FI"
        billing_address.emailAddress = "test@mannen.com"
        basic_params["object"] = billing_address

        response = self.client.update_billing_address(basic_params)

        response = self.client.add_order(basic_params)
        # Reason: Not Found
        with self.assertRaises(RuntimeError) as e:
            response = self.client.add_order(basic_params)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import validators
from .data_object import DataObject


class ContactInfo(DataObject):
    '''Data object for creating a Contact info to ePages webshop'''

    def __init__(self):
        # string The name that appears on the contact information page, e.g. Contact Us.
        self._name = None
        # string The name of the contact information page, that appears on the browser tab.
        self._title = None
        # string The name of the contact information page that appears in the navigation bar.
        self._navigationCaption = None
        # string Additional short information that can be given to e.g. better explain what’s on the contact information page.
        self._shortDescription = None
        # string Additional information that can be added to the contact information page, e.g. tax identification number or bank account.
        self._description = None
        # string The name of the shop.
        self._company = None
        # string The contact person for the shop, usually the shop owner.
        self._contactPerson = None
        # string The job title of the contact person.
        self._contactPersonJobTitle = None
        # string The postal address of the shop.
        self._address = None
        # string The phone number of the shop.
        self._phone = None
        # string The email address of the shop.
        self._email = None

    @property
    def name(self):
        return self._name

    @name.setter
    def name(self, value):
        self._name = self._check_str(value, "Name has to be a str.")

    @property
    def title(self):
        return self._title

    @title.setter
    def title(self, value):
        self._title = self._check_str(value, "Title has to be a str.")

    @property
    def navigationCaption(self):
        return self._navigationCaption

    @navigationCaption.setter
    def navigationCaption(self, value):
        self._navigationCaption = self._check_str(value, "NavigationCaption has to be a str.")

    @property
    def shortDescription(self):
        return self._shortDescription

    @shortDescription.setter
    def shortDescription(self, value):
        self._shortDescription = self._check_str(value, "ShortDescription has to be a str.")

    @property
    def description(self):
        return self._description

    @description.setter
    def description(self, value):
        self._description = self._check_str(value, "Description has to be a str.")

    @property
    def company(self):
        return self._company

    @company.setter
    def company(self, value):
        self._company = self._check_str(value, "Company has to be a str.")

    @property
    def contactPerson(self):
        return self._contactPerson

    @contactPerson.setter
    def contactPerson(self, value):
        self._contactPerson = self._check_str(value, "ContactPerson has to be a str.")

    @property
    def contactPersonJobTitle(self):
        return self._contactPersonJobTitle

    @contactPersonJobTitle.setter
    def contactPersonJobTitle(self, value):
        self._contactPersonJobTitle = self._check_str(value, "ContactPersonJobTitle has to be a str.")

    @property
    def address(self):
        return self._address

    @address.setter
    def address(self, value):
        self._address = self._check_str(value, "Address has to be a str.")

    @property
    def phone(self):
        return self._phone

    @phone.setter
    def phone(self, value):
        self._phone = self._check_str(value, "Phone has to be a str.")

    @property
    def email(self):
        return self._email

    @email.setter
    def email(self, value):
        self._email = self._check_email(value, "Email has to be a valid email address.")

    def is_valid(self):
        return True



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import unittest

# import the package
import epages_client

from epages_client.dataobjects.category_create import CategoryCreate

# import base class for unit testing
from .base_unit_test import BaseUnitTest


class TestStringMethods(BaseUnitTest):
    def setUp(self):
        pass

    def test_0001_simple_correct_inputs(self):
        category = CategoryCreate()
        # string The name of the category.
        category.name = "Cat"
        # string The unique identifier of the new category. If already in use, the system will automatically increment by number, e.g. alias1, alias2.
        category.alias = "pets_cats"
        # string The page title of the category.
        category.pageTitle = "Cats"
        # string The description of the category.
        category.description = "Products for cats"
        # string The name of the category that appears in the navigation bar.
        category.navigationCaption = "Navi Cat"

        right_answer = {
            'name': 'Cat',
            'alias': 'pets_cats',
            'pageTitle': 'Cats',
            'description': 'Products for cats',
            'navigationCaption': 'Navi Cat',
        }

        self.assertEqual(category.get_dict(), right_answer)

    def test_0002_incorrect_inputs(self):
        category = CategoryCreate()
        with self.assertRaises(TypeError) as e:
            category.name = 123456

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from .data_object import DataObject


class ListOfObjects(DataObject):
    '''Allow insert only one type items to a list'''

    def __init__(self, type_of_object):
        self._list = []
        self._type_of_object = type_of_object

    def add(self, obj):
        """Add a new item to the list. If the item is not type of set in the constructor, throw Type error"""
        if isinstance(obj, self._type_of_object):
            self._list.append(obj)
        else:
            raise TypeError(type(
                obj).__name__ + " is not instance of " + type(self._type_of_object).__name__)

    def get(self):
        """Returns a list"""
        return self._list

    def __str__(self):
        """string representation of a list"""
        return self._list.__str__()

    def get_dict(self):
        if not isinstance(self._type_of_object, tuple) and issubclass(self._type_of_object, DataObject):
            return [x.get_dict() for x in self._list]
        else:
            return self._list

    def is_valid(self):
        '''
        Check the validity of every item if item is a DataObject.
        First we have to check if item is not a tuple, because issubclass
        needs the first argument to be a class. And if type_of_object is
        a string, it is actually string_types from library six, and it is a tuple.
        '''
        if not isinstance(self._type_of_object, tuple):
            if issubclass(self._type_of_object, DataObject):
                for item in self._list:
                    if item.is_valid() == False:
                        return False
        return True



# -*- coding: utf-8 -*-
import unittest
from pprint import pprint

# import the package
import epages_client

from epages_client.dataobjects.order_update import OrderUpdate
from epages_client.dataobjects.remove_value import RemoveValue

# import base class for unit testing
from .base_unit_test import BaseUnitTest


class TestStringMethods(BaseUnitTest):
    def setUp(self):
        pass

    def test_0001_remove__values(self):
        order = OrderUpdate()
        order.customerComment = RemoveValue()
        order.internalNote = RemoveValue()
        order.viewedOn = RemoveValue()
        order.rejectedOn = RemoveValue()
        order.inProcessOn = RemoveValue()
        order.pendingOn = RemoveValue()
        order.readyForDispatchOn = RemoveValue()
        order.partiallyDispatchedOn = RemoveValue()
        order.dispatchedOn = RemoveValue()
        order.deliveredOn = RemoveValue()
        order.partiallyInvoicedOn = RemoveValue()
        order.invoicedOn = RemoveValue()
        order.partiallyPaidOn = RemoveValue()
        order.paidOn = RemoveValue()
        order.returnedOn = RemoveValue()
        order.closedOn = RemoveValue()
        order.archivedOn = RemoveValue()

        right_answer = [
            {'op': 'remove', 'path': '/deliveredOn'},
            {'op': 'remove', 'path': '/rejectedOn'},
            {'op': 'remove', 'path': '/archivedOn'},
            {'op': 'remove', 'path': '/internalNote'},
            {'op': 'remove', 'path': '/partiallyInvoicedOn'},
            {'op': 'remove', 'path': '/closedOn'},
            {'op': 'remove', 'path': '/pendingOn'},
            {'op': 'remove', 'path': '/viewedOn'},
            {'op': 'remove', 'path': '/readyForDispatchOn'},
            {'op': 'remove', 'path': '/returnedOn'},
            {'op': 'remove', 'path': '/partiallyPaidOn'},
            {'op': 'remove', 'path': '/dispatchedOn'},
            {'op': 'remove', 'path': '/partiallyDispatchedOn'},
            {'op': 'remove', 'path': '/paidOn'},
            {'op': 'remove', 'path': '/invoicedOn'},
            {'op': 'remove', 'path': '/inProcessOn'},
            {'op': 'remove', 'path': '/customerComment'}
        ]

        self.assert_count_items_equal(order.get_patch(), right_answer)

    def test_0002_change_address(self):
        order = OrderUpdate()
        order.shippingAddress.street = "Finlaysoninkuja 19"
        order.shippingAddress.zipCode = "33210"
        order.shippingAddress.city = "TAMPERE"

        right_answer = [
            {
                'op': 'add',
                'path': '/shippingAddress',
                'value': {
                    'street': "Finlaysoninkuja 19",
                    'zipCode': "33210",
                    'city': "TAMPERE"
                }
            }
        ]

        self.assertEqual(order.get_patch(), right_answer)

    def test_0003_illegals(self):
        order = OrderUpdate()
        with self.assertRaises(TypeError) as e:
            order.orderNumber = RemoveValue()


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from .data_object import DataObject
from .list_of_objects import ListOfObjects
from .product_line_item_create import ProductLineItemCreate


class CartCreate(DataObject):
    '''Data object for creating a new cart to ePages webshop'''

    def __init__(self):
        # string The currency code of the price according to ISO 4217.
        self._currency = None
        # string Indicates if the amount includes tax. Can be gross, net or none.
        self._taxType = None
        # string The locale that identifies the origin of the customer.
        self._locale = None
        # array of productLineItem (create request) The product line items in the basket.
        self.lineItems = ListOfObjects(ProductLineItemCreate)

    @property
    def currency(self):
        return self._currency

    @currency.setter
    def currency(self, value):
        self._currency = self._check_str(value, "Currency has to be a str.")

    @property
    def taxType(self):
        return self._taxType

    @taxType.setter
    def taxType(self, value):
        if value in ('GROSS', 'NET', 'NONE'):
            self._taxType = value
        else:
            raise ValueError('TaxType must be GROSS, NET or NONE (str).')

    @property
    def locale(self):
        return self._locale

    @locale.setter
    def locale(self, value):
        self._locale = self._check_str(value, "Locale has to be a str.")

    def is_valid(self):
        return self.lineItems.is_valid()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from .data_object import DataObject


class ContentPage(DataObject):
    '''Data object for creating a content page to ePages webshop'''

    def __init__(self):
        # string The name that appears on the page.
        self._name = None
        # string The name of the page, that appears on the browser tab.
        self._title = None
        # string The name of the page, that appears in the navigation bar.
        self._navigationCaption = None
        # string Additional short information that can be given to better explain what’s on the page.
        self._shortDescription = None
        # string Information on the topic of the page.
        self._description = None

    @property
    def name(self):
        return self._name

    @name.setter
    def name(self, value):
        self._name = self._check_str(value, "Name has to be a str.")

    @property
    def title(self):
        return self._title

    @title.setter
    def title(self, value):
        self._title = self._check_str(value, "Title has to be a str.")

    @property
    def navigationCaption(self):
        return self._navigationCaption

    @navigationCaption.setter
    def navigationCaption(self, value):
        self._navigationCaption = self._check_str(
            value, "NavigationCaption has to be a str.")

    @property
    def shortDescription(self):
        return self._shortDescription

    @shortDescription.setter
    def shortDescription(self, value):
        self._shortDescription = self._check_str(
            value, "ShortDescription has to be a str.")

    @property
    def description(self):
        return self._description

    @description.setter
    def description(self, value):
        self._description = self._check_str(
            value,  "Description has to be a str.")

    def is_valid(self):
        return True



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from .data_object import DataObject


class CategoryCreate(DataObject):
    '''Data object for creating a new category to ePages webshop'''

    def __init__(self):
        # string The name of the category.
        self._name = None
        # string The unique identifier of the new category. If already in use, the system will automatically increment by number, e.g. alias1, alias2.
        self._alias = None
        # string The page title of the category.
        self._pageTitle = None
        # string The description of the category.
        self._description = None
        # string The name of the category that appears in the navigation bar.
        self._navigationCaption = None

    @property
    def name(self):
        return self._name

    @name.setter
    def name(self, value):
        self._name = self._check_str(value, "Name has to be str.")

    @property
    def alias(self):
        return self._alias

    @alias.setter
    def alias(self, value):
        self._alias = self._check_str(value, "Alias has to be str.")

    @property
    def pageTitle(self):
        return self._pageTitle

    @pageTitle.setter
    def pageTitle(self, value):
        self._pageTitle = self._check_str(value, "PageTitle has to be str.")

    @property
    def description(self):
        return self._description

    @description.setter
    def description(self, value):
        self._description = self._check_str(
            value, "Description has to be str.")

    @property
    def navigationCaption(self):
        return self._navigationCaption

    @navigationCaption.setter
    def navigationCaption(self, value):
        self._navigationCaption = self._check_str(
            value, "NavigationCaption has to be str.")

    def is_valid(self):
        return True



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from six import string_types
from .data_object import DataObject
from .enum_fetch_operator import FetchOperator
from .list_of_objects import ListOfObjects
from .remove_value import RemoveValue


class ProductUpdate(DataObject):
    """Data object for updating a product"""

    def __init__(self):
        # Initialize product fields, when patching product.
        # Underscores in the middle of the variable name are replaced with slash "/"

        # string The product number (mandatory).
        self._productNumber = None
        # string The name of the product.
        self._name = None
        # string The short description of the product.
        self._shortDescription = None
        # string The description of the product.
        self._description = None
        # string The manufacturer of the product.
        self._manufacturer = None
        # string The Universal Product Code of the product.
        self._upc = None
        # string The European Article Number of the product, either EAN-8 or EAN-13.
        self._ean = None
        # string The essential features of the product.
        self._essentialFeatures = None
        # array of string The search terms for the product determined by the merchant in the administration.
        self.searchKeywords = ListOfObjects(string_types)
        # number The price of the product.
        self._priceInfo_price_amount = None
        # number The original price, if product own price is discounted
        self._priceInfo_manufacturerPrice_amount = None
        # number The deposit price for the product, e.g. bottle deposit.
        self._priceInfo_depositPrice_amount = None
        # string The average time of the product being delivered to the customer. By default, the delivery period is displayed in days, but this can be changed by the merchant.
        self._deliveryPeriod = None
        # number Indicates the minimum stock level of the product.
        self._minStocklevel = None
        # string The unique manufacturer identifier of the product.
        self._manufacturerProductNumber = None
        # number The length of the product in mm.
        self._productLength = None
        # number The width of the product in mm.
        self._productWidth = None
        # number The height of the product in mm.
        self._productHeight = None
        # string The unique identifier of the tax class.
        self._priceInfo_taxClass_taxClassId = None
        # string The name of the product image.
        self._productImage = None
        # number Indicates the stock level of the product.
        self._stocklevel = None
        # boolean Indicates if the product is displayed in the shop.
        self._visible = None

        # Available operations for fields
        self.legals = {
            '/productNumber': (FetchOperator.ADD,),
            '/name': (FetchOperator.ADD, FetchOperator.REMOVE),
            '/shortDescription': (FetchOperator.ADD, FetchOperator.REMOVE),
            '/description': (FetchOperator.ADD, FetchOperator.REMOVE),
            '/manufacturer': (FetchOperator.ADD, FetchOperator.REMOVE),
            '/upc': (FetchOperator.ADD, FetchOperator.REMOVE),
            '/ean': (FetchOperator.ADD, FetchOperator.REMOVE),
            '/essentialFeatures': (FetchOperator.ADD, FetchOperator.REMOVE),
            '/searchKeywords': (FetchOperator.ADD, FetchOperator.REMOVE),
            '/priceInfo/price/amount': (FetchOperator.ADD, FetchOperator.REMOVE),
            '/priceInfo/manufacturerPrice/amount': (FetchOperator.ADD, FetchOperator.REMOVE),
            '/priceInfo/depositPrice/amount': (FetchOperator.ADD, FetchOperator.REMOVE),
            '/deliveryPeriod': (FetchOperator.ADD, FetchOperator.REMOVE),
            '/minStocklevel': (FetchOperator.ADD, FetchOperator.REMOVE),
            '/manufacturerProductNumber': (FetchOperator.ADD, FetchOperator.REMOVE),
            '/productLength': (FetchOperator.ADD, FetchOperator.REMOVE),
            '/productWidth': (FetchOperator.ADD, FetchOperator.REMOVE),
            '/productHeight': (FetchOperator.ADD, FetchOperator.REMOVE),
            '/priceInfo/taxClass/taxClassId': (FetchOperator.REMOVE, FetchOperator.REPLACE),
            '/productImage': (FetchOperator.ADD,),
            '/stocklevel': (FetchOperator.ADD, FetchOperator.REMOVE),
            '/visible': (FetchOperator.ADD,),
        }

    @property
    def productNumber(self):
        return self._productNumber

    @productNumber.setter
    def productNumber(self, value):
        self._productNumber = self._check_str(
            value, "ProductId has to be a str.", False)

    @property
    def name(self):
        return self._name

    @name.setter
    def name(self, value):
        self._name = self._check_str(
            value, "Name has to be a str or RemoveValue.", True)

    @property
    def shortDescription(self):
        return self._shortDescription

    @shortDescription.setter
    def shortDescription(self, value):
        self._shortDescription = self._check_str(
            value, "ShortDescription has to be a str or RemoveValue.", True)

    @property
    def description(self):
        return self._description

    @description.setter
    def description(self, value):
        self._description = self._check_str(
            value, "Description has to be a str or RemoveValue.", True)

    @property
    def manufacturer(self):
        return self._manufacturer

    @manufacturer.setter
    def manufacturer(self, value):
        self._manufacturer = self._check_str(
            value, "Manufacturer has to be a str or RemoveValue.", True)

    @property
    def upc(self):
        return self._upc

    @upc.setter
    def upc(self, value):
        self._upc = self._check_str(
            value, "Upc has to be a str or RemoveValue.", True)

    @property
    def ean(self):
        return self._ean

    @ean.setter
    def ean(self, value):
        self._ean = self._check_str(
            value, "Ean has to be a str or RemoveValue.", True)

    @property
    def essentialFeatures(self):
        return self._essentialFeatures

    @essentialFeatures.setter
    def essentialFeatures(self, value):
        self._essentialFeatures = self._check_str(
            value, "EssentialFeatures has to be a str or RemoveValue.", True)

    @property
    def price(self):
        return self._priceInfo_price_amount

    @price.setter
    def price(self, value):
        self._priceInfo_price_amount = self._check_numeric(
            value, 'Price has to be a numeric type or RemoveValue.', True)

    @property
    def manufacturerPrice(self):
        return self._priceInfo_manufacturerPrice_amount

    @manufacturerPrice.setter
    def manufacturerPrice(self, value):
        self._priceInfo_manufacturerPrice_amount = self._check_numeric(
            value, 'Manufacturer price has to be a numeric type or RemoveValue.', True)

    @property
    def depositPrice(self):
        return self._priceInfo_depositPrice_amount

    @depositPrice.setter
    def depositPrice(self, value):
        self._priceInfo_depositPrice_amount = self._check_numeric(
            value, 'DepositPrice has to be a numeric type or RemoveValue.', True)

    @property
    def deliveryPeriod(self):
        return self._deliveryPeriod

    @deliveryPeriod.setter
    def deliveryPeriod(self, value):
        self._deliveryPeriod = self._check_str(
            value, "DeliveryPeriod has to be a str or RemoveValue.", True)

    @property
    def minStocklevel(self):
        return self._minStocklevel

    @minStocklevel.setter
    def minStocklevel(self, value):
        self._minStocklevel = self._check_numeric(
            value, 'Minimum stocklevel has to be a numeric type or RemoveValue.', True)

    @property
    def manufacturerProductNumber(self):
        return self._manufacturerProductNumber

    @manufacturerProductNumber.setter
    def manufacturerProductNumber(self, value):
        self._manufacturerProductNumber = self._check_str(
            value, 'ManufacturerProductNumber has to be a str or RemoveValue.', True)

    @property
    def productLength(self):
        return self._productLength

    @productLength.setter
    def productLength(self, value):
        self._productLength = self._check_numeric(
            value, 'Product length has to be a numeric type or RemoveValue.', True)

    @property
    def productWidth(self):
        return self._productWidth

    @productWidth.setter
    def productWidth(self, value):
        self._productWidth = self._check_numeric(
            value, 'Product width has to be a numeric type or RemoveValue.', True)

    @property
    def productHeight(self):
        return self._productHeight

    @productHeight.setter
    def productHeight(self, value):
        self._productHeight = self._check_numeric(
            value, 'Product height has to be a numeric type or RemoveValue.', True)

    @property
    def taxClassId(self):
        return self._priceInfo_taxClass_taxClassId

    @taxClassId.setter
    def taxClassId(self, value):
        self._priceInfo_taxClass_taxClassId = self._check_str(
            value, 'TaxClassId has to be a numeric type or RemoveValue.', True)

    @property
    def productImage(self):
        return self._productImage

    @productImage.setter
    def productImage(self, value):
        self._productImage = self._check_str(
            value, "ProductImage has to be a str.", False)

    @property
    def stocklevel(self):
        return self._stocklevel

    @stocklevel.setter
    def stocklevel(self, value):
        self._stocklevel = self._check_numeric(
            value, 'Stocklevel has to be a numeric type or RemoveValue.', True)

    @property
    def visible(self):
        return self._visible

    @visible.setter
    def visible(self, value):
        self._visible = self._check_bool(
            value, 'Visible has to be a bool.', False)

    def get_patch(self):
        return self.get_list_of_json_patches(self.legals)

    def is_valid(self):
        return self.searchKeywords.is_valid()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import os
import uuid

# import the RestClient class
from epages_client.client import RestClient

# Import the customer create class
from epages_client.dataobjects.customer_create import CustomerCreate

# Import the customer patch class
from epages_client.dataobjects.customer_update import CustomerUpdate

# import base class for unit testing
from .base_unit_test import BaseUnitTest


class TestCustomerMethods(BaseUnitTest):
    '''A class for testing customer related methods on RestClient class'''

    def setUp(self):

        self.client = RestClient(
            os.environ["EPAGES_API_URL"], os.environ["EPAGES_API_TOKEN"])

        self.params = {
            "query": {},
            "param1": "",
            "param2": ""
        }

    def test_001_add_customer_no_object(self):

        with self.assertRaises(RuntimeError) as e:
            response = self.client.add_customer(self.params)

    def test_002_add_customer_invalid_object(self):

        self.params["object"] = None

        with self.assertRaises(TypeError) as e:
            response = self.client.add_customer(self.params)

    def test_003_add_customer_empty_billing_address(self):

        customer = CustomerCreate()

        self.params["object"] = customer

        with self.assertRaises(ValueError) as e:
            response = self.client.add_customer(self.params)

    def test_004_add_customer(self):

        customer = CustomerCreate()
        customer.billingAddress.firstName = "John"
        customer.billingAddress.lastName = "Doe"
        customer.billingAddress.emailAddress = "john@whiners.co.uk"

        self.params["object"] = customer

        response = self.client.add_customer(self.params)

        self.assertEqual(isinstance(response, dict), True)

    def test_005_get_customers(self):

        customers = self.client.get_customers(self.params)

        self.assertEqual(isinstance(customers, dict), True)

    def test_006_get_customer_no_id(self):

        with self.assertRaises(ValueError) as e:
            customer = self.client.get_customer(self.params)

    def test_007_get_customer_false_id(self):

        self.params["param1"] = str(uuid.uuid4())

        with self.assertRaises(RuntimeError) as e:
            customer = self.client.get_customer(self.params)

    def test_008_get_customer_correct_id(self):

        customers = self.client.get_customers(self.params)

        customer_id = customers["items"][1]["customerId"]

        self.params["param1"] = customer_id

        customer = self.client.get_customer(self.params)

        self.assertEqual(isinstance(customer, dict), True)

    def test_009_update_customer_no_id(self):

        with self.assertRaises(ValueError) as e:
            response = self.client.update_product(self.params)

    def test_010_update_customer_no_object(self):

        customers = self.client.get_customers(self.params)

        customer_id = customers["items"][1]["customerId"]

        self.params["param1"] = customer_id

        with self.assertRaises(RuntimeError) as e:
            response = self.client.update_customer(self.params)

    def test_011_update_customer_invalid_object(self):

        customers = self.client.get_customers(self.params)

        customer_id = customers["items"][1]["customerId"]

        self.params["param1"] = customer_id

        self.params["object"] = None

        with self.assertRaises(TypeError) as e:
            response = self.client.update_customer(self.params)

    def test_012_update_customer(self):

        customers = self.client.get_customers(self.params)

        customer_id = customers["items"][1]["customerId"]

        self.params["param1"] = customer_id

        customer = CustomerUpdate()
        customer.internalNote = "This customer whines a lot."

        self.params["object"] = customer

        response = self.client.update_customer(self.params)

        self.assertEqual(isinstance(response, dict), True)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import os
import uuid

# import the RestClient class
from epages_client.client import RestClient

# import base class for unit testing
from .base_unit_test import BaseUnitTest


class TestNewsletterMethods(BaseUnitTest):
    '''A class for testing newsletter related methods on RestClient class'''

    def setUp(self):

        self.client = RestClient(
            os.environ["EPAGES_API_URL"], os.environ["EPAGES_API_TOKEN"])

        self.params = {
            "query": {},
            "param1": "",
            "param2": ""
        }

    def test_001_get_newsletter_campaigns(self):

        newsletter_campaigns = self.client.get_newsletter_campaigns(
            self.params)

        self.assertEqual(isinstance(newsletter_campaigns, dict), True)

    def test_002_get_newsletter_campaign_subscribers_no_id(self):

        with self.assertRaises(ValueError) as e:
            newsletter_subscribers = self.client.get_newsletter_campaign_subscribers(
                self.params)

    def test_003_get_newsletter_campaign_subscribers_false_id(self):

        self.params["param1"] = str(uuid.uuid4())

        with self.assertRaises(RuntimeError) as e:
            newsletter_subscribers = self.client.get_newsletter_campaign_subscribers(
                self.params)

    def test_004_get_newsletter_campaign_subscribers(self):

        newsletter_campaigns = self.client.get_newsletter_campaigns(
            self.params)

        # If there are some newsletters, check if the first one has subscribers
        if newsletter_campaigns["results"] > 0:

            campaign_id = newsletter_campaigns["items"][0]["campaignId"]
            self.params["param1"] = campaign_id

            newsletter_subscribers = self.client.get_newsletter_campaign_subscribers(
                self.params)

            self.assertEqual(isinstance(newsletter_subscribers, dict), True)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import os
import time

# import the RestClient class
from epages_client.client import RestClient

# import base class for unit testing
from .base_unit_test import BaseUnitTest


class TestClient(BaseUnitTest):
    '''A class for testing the RestClient class'''

    def setUp(self):

        self.client = RestClient(
            os.environ["EPAGES_API_URL"], os.environ["EPAGES_API_TOKEN"])

        self.params = {
            "query": {},
            "param1": "",
            "param2": ""
        }

    def test_001_use_undefined_command(self):

        with self.assertRaises(TypeError) as e:
            response = self.client.foobar()

    def test_002_set_and_get_currency(self):

        self.client.currency = "EUR"

        self.assertEqual(self.client.currency, "EUR")

    def test_003_set_and_get_locale(self):

        self.client.locale = "fi_FI"

        self.assertEqual(self.client.locale, "fi_FI")

    def test_004_method_mapping_not_found(self):

        # Get current directory
        dir_path = os.path.dirname(os.path.realpath(__file__))

        # Get one directory up, that is basically joining '..'
        # to the path and getting absolute path from it
        one_dir_up = os.path.abspath(os.path.join(dir_path, ".."))

        # Add the project name to the path to form the client directory
        client_dir = os.path.join(one_dir_up, "epages_client")

        # Create paths for correct and incorrect method mapping files
        correct_mapping = os.path.join(client_dir, "method_mapping.json")
        incorrect_mapping = os.path.join(client_dir, "method_rapping.json")

        # Rename method mapping to be a invalid one
        os.rename(correct_mapping, incorrect_mapping)

        # Reload client
        self.client = RestClient(
            os.environ["EPAGES_API_URL"], os.environ["EPAGES_API_TOKEN"])

        # When the method mapping file is not found, and the called method
        # is not found either, there raises a TypeError
        with self.assertRaises(TypeError) as e:
            self.client.get_shop_info()

        # Rename the method mapping file to its original name
        os.rename(incorrect_mapping, correct_mapping)

        time.sleep(1)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from .data_object import DataObject


class CategoryUpdate(DataObject):
    '''Data object for updating a category to ePages webshop'''

    def __init__(self):
        # string The unique identifier of the category a product is assigned to.
        self._categoryId = None
        # string The name of the category.
        self._name = None
        # string The unique identifier of the category.
        self._alias = None
        # string The page title of this category.
        self._pageTitle = None
        # string The description of the category.
        self._description = None
        # string The name of the category page that appears in the navigation bar.
        self._navigationCaption = None
        # boolean Indicates if the category is displayed in the shop.
        self._visible = None

    @property
    def categoryId(self):
        return self._categoryId

    @categoryId.setter
    def categoryId(self, value):
        self._categoryId = self._check_str(
            value, "CategoryId has to be a str.")

    @property
    def name(self):
        return self._name

    @name.setter
    def name(self, value):
        self._name = self._check_str(value, "Name has to be a str.")

    @property
    def alias(self):
        return self._alias

    @alias.setter
    def alias(self, value):
        self._alias = self._check_str(value, "Alias has to be a str.")

    @property
    def pageTitle(self):
        return self._pageTitle

    @pageTitle.setter
    def pageTitle(self, value):
        self._pageTitle = self._check_str(value, "PageTitle has to be a str.")

    @property
    def description(self):
        return self._description

    @description.setter
    def description(self, value):
        self._description = self._check_str(
            value, "Description has to be a str.")

    @property
    def navigationCaption(self):
        return self._navigationCaption

    @navigationCaption.setter
    def navigationCaption(self, value):
        self._navigationCaption = self._check_str(
            value, "NavigationCaption has to be a str.")

    @property
    def visible(self):
        return self._visible

    @visible.setter
    def visible(self, value):
        self._visible = self._check_bool(value, 'Visible has to be a bool.')

    def is_valid(self):

        # Category id and alias must be sent always when updating
        if not self._categoryId or not self._alias:
            return False
        else:
            return True



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

"""Tests for `epages_client.dataobjects.product_create` package."""
import unittest

# import the package
import epages_client

from epages_client.dataobjects.product_update import ProductUpdate
from epages_client.dataobjects.remove_value import RemoveValue

# import base class for unit testing
from .base_unit_test import BaseUnitTest


class TestStringMethods(BaseUnitTest):
    def setUp(self):
        pass

    def test_1001_correct_inputs(self):
        product = ProductUpdate()

        product.productNumber = "prod_1"
        self.assertEqual(product.productNumber, "prod_1")

        product.name = "Deddy Bear"
        self.assertEqual(product.name, "Deddy Bear")

        product.shortDescription = "Soft Deddy Bear"
        self.assertEqual(product.shortDescription, "Soft Deddy Bear")

        product.description = "Tom’s 18″ Teddy Bear"
        self.assertEqual(product.description, "Tom’s 18″ Teddy Bear")

        product.manufacturer = "Christopher Robin"
        self.assertEqual(product.manufacturer, "Christopher Robin")

        product.upc = "9501101530003"
        self.assertEqual(product.upc, "9501101530003")

        product.ean = "0012345678905"
        self.assertEqual(product.ean, "0012345678905")

        product.essentialFeatures = "yellow eyes"
        self.assertEqual(product.essentialFeatures, "yellow eyes")

        product.price = 29.99
        self.assertEqual(product.price, 29.99)

        product.manufacturerPrice = 49.90
        self.assertEqual(product.manufacturerPrice, 49.90)

        product.depositPrice = 0.00
        self.assertEqual(product.depositPrice, 0.00)

        product.deliveryPeriod = "5 days"
        self.assertEqual(product.deliveryPeriod, "5 days")

        product.minStocklevel = 6
        self.assertEqual(product.minStocklevel, 6)

        product.manufacturerProductNumber = "00000001ZDEDDY"
        self.assertEqual(product.manufacturerProductNumber, "00000001ZDEDDY")

        product.productLength = 50
        self.assertEqual(product.productLength, 50)

        product.productWidth = 60
        self.assertEqual(product.productWidth, 60)

        product.productHeight = 457
        self.assertEqual(product.productHeight, 457)

        product.taxClassId = "tax01"
        self.assertEqual(product.taxClassId, "tax01")

        product.productImage = "teddy0001_big.jpg"
        self.assertEqual(product.productImage, "teddy0001_big.jpg")

        product.stocklevel = 5
        self.assertEqual(product.stocklevel, 5)

        product.visible = True
        self.assertEqual(product.visible, True)

        product.searchKeywords.add('deddy')
        self.assertEqual(product.searchKeywords.get(), ['deddy'])

        product.searchKeywords.add('toy')
        self.assertEqual(product.searchKeywords.get(), ['deddy', 'toy'])

        patch_values = [
            {'op': 'replace', 'path': '/priceInfo/taxClass/taxClassId', 'value': 'tax01'},
            {'op': 'add', 'path': '/visible', 'value': True},
            {'op': 'add', 'path': '/productNumber', 'value': 'prod_1'},
            {'op': 'add', 'path': '/productImage', 'value': 'teddy0001_big.jpg'},
            {'op': 'add', 'path': '/deliveryPeriod', 'value': '5 days'},
            {'op': 'add', 'path': '/essentialFeatures', 'value': 'yellow eyes'},
            {'op': 'add', 'path': '/description', 'value': 'Tom’s 18″ Teddy Bear'},
            {'op': 'add', 'path': '/shortDescription', 'value': 'Soft Deddy Bear'},
            {'op': 'add', 'path': '/productWidth', 'value': 60},
            {'op': 'add', 'path': '/upc', 'value': '9501101530003'},
            {'op': 'add', 'path': '/searchKeywords',
                'value': ['deddy', 'toy']},
            {'op': 'add', 'path': '/priceInfo/manufacturerPrice/amount', 'value': 49.9},
            {'op': 'add', 'path': '/manufacturerProductNumber',
                'value': '00000001ZDEDDY'},
            {'op': 'add', 'path': '/priceInfo/price/amount', 'value': 29.99},
            {'op': 'add', 'path': '/productLength', 'value': 50},
            {'op': 'add', 'path': '/name', 'value': 'Deddy Bear'},
            {'op': 'add', 'path': '/manufacturer', 'value': 'Christopher Robin'},
            {'op': 'add', 'path': '/ean', 'value': '0012345678905'},
            {'op': 'add', 'path': '/priceInfo/depositPrice/amount', 'value': 0.0},
            {'op': 'add', 'path': '/stocklevel', 'value': 5},
            {'op': 'add', 'path': '/productHeight', 'value': 457},
            {'op': 'add', 'path': '/minStocklevel', 'value': 6}
        ]

        self.assert_count_items_equal(product.get_patch(), patch_values)

    def test_1201_correct_removes(self):
        product = ProductUpdate()
        product.name = RemoveValue()

        # test str
        patch_value = [{'op': 'remove', 'path': '/name'}]
        self.assert_count_items_equal(product.get_patch(), patch_value)

        # test numeric
        product.minStocklevel = RemoveValue()
        patch_value = [{'op': 'remove', 'path': '/name'},
                       {'op': 'remove', 'path': '/minStocklevel'}]
        self.assert_count_items_equal(product.get_patch(), patch_value)

        # ListOfObject - user overwrites when he/she wants to remove values
        product.searchKeywords = RemoveValue()
        self.assertIsInstance(product.searchKeywords, RemoveValue)
        patch_value = [{'op': 'remove', 'path': '/name'}, {'op': 'remove',
                                                           'path': '/minStocklevel'}, {'op': 'remove', 'path': "/searchKeywords"}]
        self.assert_count_items_equal(product.get_patch(), patch_value)

    def test_2000_invalid_inputs(self):
        product = ProductUpdate()

        with self.assertRaises(TypeError) as e:
            product.price = "12.21 €"

        with self.assertRaises(TypeError) as e:
            product.visible = "True"


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from .data_object import DataObject


class ProductLineItemUpdate(DataObject):
    """Data object for ProductLineItemUpdate"""

    def __init__(self, quantity=None):
        self._quantity = None if quantity == None else self._check_numeric(
            quantity, "Quantity has to be a numeric type.")

    @property
    def quantity(self):
        return self._quantity

    @quantity.setter
    def quantity(self, value):
        self._quantity = self._check_numeric(
            value, "Quantity has to be a numeric type.")

    def is_valid(self):
        return self._quantity != None



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import unittest
from pprint import pprint

# import the package
import epages_client

from epages_client.dataobjects.address import Address

# import base class for unit testing
from .base_unit_test import BaseUnitTest


class TestStringMethods(BaseUnitTest):
    def setUp(self):
        pass

    def test_0001_simple_correct_inputs(self):
        address = Address()
        address.company = "Company"
        address.salutation = "Mr"
        address.title = "Agent"
        address.firstName = "James"
        address.lastName = "Körri"
        address.street = "Finlaysoninkuja 19"
        address.streetDetails = ""
        address.zipCode = "33210"
        address.city = "TAMPERE"
        address.state = "Pirkanmaa"
        address.country = "Finland"
        address.vatId = "vat_01"
        address.birthday = "04.04.2014"
        address.emailAddress = "invalid@email.adress"
        address.addressExtension = "C 2"
        address.bankAccountHolder = "James Körri"
        address.bankAccountNumber = "123456-789"
        address.bankName = "Big Bank"
        address.bankSortCode = "BB"
        address.businessEmailAddress = "email.address@business.invalid"
        address.businessPhoneNumber = "+358441111111"
        address.department = "IT"
        address.displayName = "James Körri"
        address.doorCode = "0000"
        address.faxNumber = "+35845000000"
        address.fiscalCode = "000000-000"
        address.gender = "MALE"
        address.jobTitle = "Secret Agent"
        address.middleName = "Kalle"
        address.mobilePhoneNumber = "+35844000000000"
        address.phoneNumber = "+3584413579"
        address.privateEmailAddress = "private@email.address.invalid"
        address.privatePhoneNumber = "+358441234567"
        address.websiteUrl = "http://websites.invalid"

        right_answer = {
            'company': 'Company',
            'salutation': 'Mr',
            'title': 'Agent',
            'firstName': 'James',
            'lastName': 'Körri',
            'street': 'Finlaysoninkuja 19',
            'streetDetails': '',
            'zipCode': '33210',
            'city': 'TAMPERE',
            'state': 'Pirkanmaa',
            'country': 'Finland',
            'vatId': 'vat_01',
            'birthday': '04.04.2014',
            'emailAddress': 'invalid@email.adress',
            'addressExtension': 'C 2',
            'bankAccountHolder': 'James Körri',
            'bankAccountNumber': '123456-789',
            'bankName': 'Big Bank',
            'bankSortCode': 'BB',
            'businessEmailAddress': 'email.address@business.invalid',
            'businessPhoneNumber': '+358441111111',
            'department': 'IT',
            'displayName': 'James Körri',
            'doorCode': '0000',
            'faxNumber': '+35845000000',
            'fiscalCode': '000000-000',
            'gender': 'MALE',
            'jobTitle': 'Secret Agent',
            'middleName': 'Kalle',
            'mobilePhoneNumber': '+35844000000000',
            'phoneNumber': '+3584413579',
            'privateEmailAddress': 'private@email.address.invalid',
            'privatePhoneNumber': '+358441234567',
            'websiteUrl': 'http://websites.invalid'
        }

        self.assertEqual(address.get_dict(), right_answer)

    def test_0002_incorrect_email_addresses(self):
        address = Address()

        with self.assertRaises(ValueError) as e:
            address.emailAddress = 'test(at)invalid.email'

        with self.assertRaises(ValueError) as e:
            address.businessEmailAddress = 'test(at)invalid.email'

        with self.assertRaises(ValueError) as e:
            address.privateEmailAddress = 'test(at)invalid.email'

    def test_0003_incorrect_website_url(self):
        address = Address()
        with self.assertRaises(ValueError) as e:
            address.websiteUrl = "www.website.invalid"


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import os
import unittest

# Check if environment variables are set
envcheck = unittest.skipUnless(
    os.environ.get('EPAGES_API_URL', False) and
    os.environ.get('EPAGES_API_TOKEN', False), 'Environment variables not set'
)


@envcheck
class BaseUnitTest(unittest.TestCase):

    def assert_count_items_equal(self, param_1, param_2):
        '''
        A function to use either assertCountEqual or assertItemsEqual
        depending on Python version.
        '''

        try:
            # Python 3.x
            self.assertCountEqual(param_1, param_2)
        except AttributeError:
            # Python 2.x
            self.assertItemsEqual(param_1, param_2)

    def get_resources_path(self):
        # Return path for resource files

        # Get the directory where this file is located
        dir_path = os.path.dirname(os.path.realpath(__file__))

        # Set the resources directory where id files and test image are located
        resources_path = os.path.join(dir_path, "resources")

        # If the resources directory doesn't exist, raise an error
        if not os.path.exists(resources_path):
            raise OSError("Resources directory %s not found." %
                          (resources_path,))

        return resources_path

    def save_resource(self, filename, content, overwrite=True):
        # Save file to resource path

        file_path = os.path.join(self.get_resources_path(), filename)

        if overwrite == False and os.path.isfile(file_path):
            raise RuntimeError("File %s already exists" % (file_path,))

        try:
            fh = open(file_path, "w")
            fh.write(content)
            fh.close()
        except IOError:
            print("File %s couldn't be created." % (file_path,))

    def get_resource(self, filename):

        file_path = os.path.join(self.get_resources_path(), filename)

        if os.path.isfile(file_path) == False:
            raise RuntimeError("File %s not found" % (file_path,))

        with open(file_path, 'r') as read_file:
            content = read_file.read()

        return content



# -*- coding: utf-8 -*-
import unittest
from pprint import pprint

# import the package
import epages_client

from epages_client.dataobjects.customer_update import CustomerUpdate
from epages_client.dataobjects.remove_value import RemoveValue

# import base class for unit testing
from .base_unit_test import BaseUnitTest


class TestStringMethods(BaseUnitTest):
    def setUp(self):
        pass

    def test_0001_simple_correct_inputs(self):
        james = CustomerUpdate()
        james.internalNote = "WOW!"
        james.customerNumber = '007'
        self.assert_count_items_equal(james.get_patch(), [
                                      {'op': 'add', 'value': 'WOW!', 'path': '/internalNote'}, {'op': 'add', 'value': '007', 'path': '/customerNumber'}])

    def test_0002_correct_inputs(self):
        james = CustomerUpdate()
        james.internalNote = "WOW!"
        james.customerNumber = '007'
        james.billingAddress.birthday = '2018-01-02'
        james.billingAddress.gender = 'MALE'

        self.assert_count_items_equal(james.get_patch(), [{'op': 'add', 'value': 'WOW!', 'path': '/internalNote'}, {
                                      'op': 'add', 'value': '007', 'path': '/customerNumber'}, {'op': 'add', 'path': '/billingAddress', 'value': {'birthday': '2018-01-02', 'gender': 'MALE'}}])

    def test_0003_remove_values(self):
        james = CustomerUpdate()
        james.internalNote = RemoveValue()
        james.billingAddress = RemoveValue()

        correct_answer = [
            {'op': 'remove', 'path': '/billingAddress'},
            {'op': 'remove', 'path': '/internalNote'}
        ]

        self.assert_count_items_equal(james.get_patch(), correct_answer)


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import unittest
from pprint import pprint

# import the package
import epages_client

# import the RestClient class
from epages_client.client import RestClient

from epages_client.dataobjects.cart_create import CartCreate
from epages_client.dataobjects.product_line_item_create import ProductLineItemCreate

# import base class for unit testing
from .base_unit_test import BaseUnitTest


class TestProductMethods(BaseUnitTest):
    '''A class for testing dataobject CartCreate'''

    def setUp(self):
        self.cart = CartCreate()

    def test_001_set_basic_data(self):

        self.cart.currency = 'EUR'
        self.cart.taxType = 'NET'
        self.cart.locale = 'fi_fi'

        right_answer = {
            'currency': 'EUR',
            'taxType': 'NET',
            'locale': 'fi_fi'
        }

        self.assertEqual(self.cart.get_dict(), right_answer)

    def test_002_add_Products(self):

        self.cart.currency = 'EUR'
        self.cart.taxType = 'NET'
        self.cart.locale = 'fi_fi'

        product = ProductLineItemCreate()
        product.productId = '00001Z'
        product.quantity = 5
        self.cart.lineItems.add(product)

        right_answer = {
            'currency': 'EUR',
            'taxType': 'NET',
            'locale': 'fi_fi',
            'lineItems': [
                {
                    'productId': '00001Z',
                    'quantity': 5
                }
            ]
        }

        self.assertEqual(self.cart.get_dict(), right_answer)

    def test_003_invalid_input_to_lineItems(self):
        with self.assertRaises(TypeError) as e:
            self.cart.lineItems.add('Product One')

    def tearDown(self):
        pass



# -*- coding: utf-8 -*-
"""Tests for `epages_client.dataobjects.product_line_item_update` package."""
import unittest

# import the package
import epages_client

from epages_client.dataobjects.product_line_item_update import ProductLineItemUpdate

# import base class for unit testing
from .base_unit_test import BaseUnitTest


class TestStringMethods(BaseUnitTest):
    def test_0001_test_property(self):
        line = ProductLineItemUpdate()
        line.quantity = 5
        self.assertEqual(line.get_dict(), {"quantity": 5})

    def test_0002_test_init_set(self):
        line = ProductLineItemUpdate(3.3)
        self.assertEqual(line.get_dict(), {"quantity": 3.3})

    def test_1001_invalid_inputs(self):
        line = ProductLineItemUpdate()
        with self.assertRaises(TypeError) as e:
            line.quantity = "4.0"

    def test_1002_invalid_init_set(self):
        line = ProductLineItemUpdate()
        with self.assertRaises(TypeError) as e:
            line.quantity = "2"


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from .data_object import DataObject
from .enum_fetch_operator import FetchOperator
from .list_of_objects import ListOfObjects
from .address import Address


class OrderUpdate(DataObject):
    """Dataobject for updating order"""

    def __init__(self):
        self._orderNumber = None
        self._customerComment = None
        self._internalNote = None
        self.billingAddress = Address()
        self.shippingAddress = Address()
        self._viewedOn = None
        self._rejectedOn = None
        self._inProcessOn = None
        self._pendingOn = None
        self._readyForDispatchOn = None
        self._partiallyDispatchedOn = None
        self._dispatchedOn = None
        self._deliveredOn = None
        self._partiallyInvoicedOn = None
        self._invoicedOn = None
        self._partiallyPaidOn = None
        self._paidOn = None
        self._returnedOn = None
        self._closedOn = None
        self._archivedOn = None

        self.legals = {
            '/orderNumber': (FetchOperator.ADD,),
            '/customerComment': (FetchOperator.ADD, FetchOperator.REMOVE),
            '/internalNote': (FetchOperator.ADD, FetchOperator.REMOVE),
            '/billingAddress': (FetchOperator.ADD,),
            '/shippingAddress': (FetchOperator.ADD,),
            '/viewedOn': (FetchOperator.ADD, FetchOperator.REMOVE),
            '/rejectedOn': (FetchOperator.ADD, FetchOperator.REMOVE),
            '/inProcessOn': (FetchOperator.ADD, FetchOperator.REMOVE),
            '/pendingOn': (FetchOperator.ADD, FetchOperator.REMOVE),
            '/readyForDispatchOn': (FetchOperator.ADD, FetchOperator.REMOVE),
            '/partiallyDispatchedOn': (FetchOperator.ADD, FetchOperator.REMOVE),
            '/dispatchedOn': (FetchOperator.ADD, FetchOperator.REMOVE),
            '/deliveredOn': (FetchOperator.ADD, FetchOperator.REMOVE),
            '/partiallyInvoicedOn': (FetchOperator.ADD, FetchOperator.REMOVE),
            '/invoicedOn': (FetchOperator.ADD, FetchOperator.REMOVE),
            '/partiallyPaidOn': (FetchOperator.ADD, FetchOperator.REMOVE),
            '/paidOn': (FetchOperator.ADD, FetchOperator.REMOVE),
            '/returnedOn': (FetchOperator.ADD, FetchOperator.REMOVE),
            '/closedOn': (FetchOperator.ADD, FetchOperator.REMOVE),
            '/archivedOn': (FetchOperator.ADD, FetchOperator.REMOVE),
        }

    @property
    def orderNumber(self):
        return self._orderNumber

    @orderNumber.setter
    def orderNumber(self, value):
        self._orderNumber = self._check_str(
            value, 'OrderNumber has to be a str.', False)

    @property
    def customerComment(self):
        return self._customerComment

    @customerComment.setter
    def customerComment(self, value):
        self._customerComment = self._check_str(
            value, 'CustomerComment has to be a str or RemoveValue.', True)

    @property
    def internalNote(self):
        return self._internalNote

    @internalNote.setter
    def internalNote(self, value):
        self._internalNote = self._check_str(
            value, 'InternalNote has to be a str or RemoveValue.', True)

    @property
    def viewedOn(self):
        return self._viewedOn

    @viewedOn.setter
    def viewedOn(self, value):
        self._viewedOn = self._check_str(
            value, 'ViewedOn has to be a str or RemoveValue.', True)

    @property
    def rejectedOn(self):
        return self._rejectedOn

    @rejectedOn.setter
    def rejectedOn(self, value):
        self._rejectedOn = self._check_str(
            value, 'RejectedOn has to be a str or RemoveValue.', True)

    @property
    def inProcessOn(self):
        return self._inProcessOn

    @inProcessOn.setter
    def inProcessOn(self, value):
        self._inProcessOn = self._check_str(
            value, 'InProcessOn has to be a str or RemoveValue.', True)

    @property
    def pendingOn(self):
        return self._pendingOn

    @pendingOn.setter
    def pendingOn(self, value):
        self._pendingOn = self._check_str(
            value, 'PendingOn has to be a str or RemoveValue.', True)

    @property
    def readyForDispatchOn(self):
        return self._readyForDispatchOn

    @readyForDispatchOn.setter
    def readyForDispatchOn(self, value):
        self._readyForDispatchOn = self._check_str(
            value, 'ReadyForDispatchOn has to be a str or RemoveValue.', True)

    @property
    def partiallyDispatchedOn(self):
        return self._partiallyDispatchedOn

    @partiallyDispatchedOn.setter
    def partiallyDispatchedOn(self, value):
        self._partiallyDispatchedOn = self._check_str(
            value, 'PartiallyDispatchedOn has to be a str or RemoveValue', True)

    @property
    def dispatchedOn(self):
        return self._dispatchedOn

    @dispatchedOn.setter
    def dispatchedOn(self, value):
        self._dispatchedOn = self._check_str(
            value, 'DispatchedOn has to be a str or RemoveValue.', True)

    @property
    def deliveredOn(self):
        return self._deliveredOn

    @deliveredOn.setter
    def deliveredOn(self, value):
        self._deliveredOn = self._check_str(
            value, 'DeliveredOn has to be a str or RemoveValue.', True)

    @property
    def partiallyInvoicedOn(self):
        return self._partiallyInvoicedOn

    @partiallyInvoicedOn.setter
    def partiallyInvoicedOn(self, value):
        self._partiallyInvoicedOn = self._check_str(
            value, 'PartiallyInvoicedOn has to be a str or RemoveValue.', True)

    @property
    def invoicedOn(self):
        return self._invoicedOn

    @invoicedOn.setter
    def invoicedOn(self, value):
        self._invoicedOn = self._check_str(
            value, 'InvoicedOn has to be a str or RemoveValue.', True)

    @property
    def partiallyPaidOn(self):
        return self._partiallyPaidOn

    @partiallyPaidOn.setter
    def partiallyPaidOn(self, value):
        self._partiallyPaidOn = self._check_str(
            value, 'PartiallyPaidOn has to be a str or RemoveValue.', True)

    @property
    def paidOn(self):
        return self._paidOn

    @paidOn.setter
    def paidOn(self, value):
        self._paidOn = self._check_str(
            value, 'PaidOn has to be a str or RemoveValue.', True)

    @property
    def returnedOn(self):
        return self._returnedOn

    @returnedOn.setter
    def returnedOn(self, value):
        self._returnedOn = self._check_str(
            value, 'ReturnedOn has to be a str or RemoveValue.', True)

    @property
    def closedOn(self):
        return self._closedOn

    @closedOn.setter
    def closedOn(self, value):
        self._closedOn = self._check_str(
            value, 'ClosedOn has to be a str or RemoveValue.', True)

    @property
    def archivedOn(self):
        return self._archivedOn

    @archivedOn.setter
    def archivedOn(self, value):
        self._archivedOn = self._check_str(
            value, 'ArchivedOn has to be a str or RemoveValue.', True)

    def is_valid(self):
        return True

    def get_patch(self):
        return self.get_list_of_json_patches(self.legals)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from six import string_types
from .data_object import DataObject
from .list_of_objects import ListOfObjects


class ProductCreate(DataObject):
    ''''Data object for create a new product to ePages webshop'''

    def __init__(self):
        # string The product number (mandatory).
        self._productNumber = None
        # string The name of the product.
        self._name = None
        # string The short description of the product.
        self._shortDescription = None
        # string The description of the product.
        self._description = None
        # string The manufacturer of the product.
        self._manufacturer = None
        # number The price of the product.
        self._price = None
        # string The essential features of the product.
        self._essentialFeatures = None
        # string The Universal Product Code of the product.
        self._upc = None
        # string The European Article Number of the product, either EAN-8 or EAN-13.
        self._ean = None
        # string The average time of the product being delivered to the customer. By default, the delivery period is displayed in days, but this can be changed by the merchant.
        self._deliveryPeriod = None
        # array of string The search terms for the product determined by the merchant in the administration.
        self.searchKeywords = ListOfObjects(string_types)
        # boolean Indicates if the product is displayed in the shop.
        self._visible = None
        # string The unique identifier of the tax class.
        self._taxClassId = None
        # number Indicates the stock level of the product.
        self._stocklevel = None
        # number The deposit price for the product, e.g. bottle deposit.
        self._depositPrice = None
        # number The sales price recommended by the manufacturer.
        self._manufacturerPrice = None

    @property
    def productNumber(self):
        return self._productNumber

    @productNumber.setter
    def productNumber(self, value):
        self._productNumber = self._check_str(
            value, "ProductNumber has to be a str.")

    @property
    def name(self):
        return self._name

    @name.setter
    def name(self, value):
        self._name = self._check_str(value, "Name has to be a str.")

    @property
    def shortDescription(self):
        return self._shortDescription

    @shortDescription.setter
    def shortDescription(self, value):
        self._shortDescription = self._check_str(
            value, "ShortDescription has to be a str.")

    @property
    def description(self):
        return self._description

    @description.setter
    def description(self, value):
        self._description = self._check_str(value, "Description has to be a str.")

    @property
    def manufacturer(self):
        return self._manufacturer

    @manufacturer.setter
    def manufacturer(self, value):
        self._manufacturer = self._check_str(value, "Manufacturer has to be a str.")

    @property
    def price(self):
        return self._price

    @price.setter
    def price(self, value):
        self._price = self._check_numeric(
            value, "Price has to be a numeric type.")

    @property
    def essentialFeatures(self):
        return self._essentialFeatures

    @essentialFeatures.setter
    def essentialFeatures(self, value):
        self._essentialFeatures = self._check_str(value, "EssentialFeatures has to be a str.")

    @property
    def upc(self):
        return self._upc

    @upc.setter
    def upc(self, value):
        self._upc = self._check_str(value, "Upc has to be a str.")

    @property
    def ean(self):
        return self._ean

    @ean.setter
    def ean(self, value):
        self._ean = self._check_str(value, 'Ean has to be a str.')

    @property
    def deliveryPeriod(self):
        return self._deliveryPeriod

    @deliveryPeriod.setter
    def deliveryPeriod(self, value):
        self._deliveryPeriod = self._check_str(value, 'DeliveryPeriod has to be a str.')

    @property
    def visible(self):
        return self._visible

    @visible.setter
    def visible(self, value):
        self._visible = self._check_bool(value, 'Visible has to be a bool type.')

    @property
    def taxClassId(self):
        return self._taxClassId

    @taxClassId.setter
    def taxClassId(self, value):
        self._taxClassId = self._check_str(value, 'TaxClassId has to be a str.')

    @property
    def stocklevel(self):
        return self._stocklevel

    @stocklevel.setter
    def stocklevel(self, value):
        self._stocklevel = self._check_numeric(
            value, 'Stocklevel has to be a numeric type.')

    @property
    def depositPrice(self):
        return self._depositPrice

    @depositPrice.setter
    def depositPrice(self, value):
        self._depositPrice = self._check_numeric(
            value, 'DepositPrice has to be a numeric type.')

    @property
    def manufacturerPrice(self):
        return self._manufacturerPrice

    @manufacturerPrice.setter
    def manufacturerPrice(self, value):
        self._manufacturerPrice = self._check_numeric(
            value, 'Stocklevel has to be a numeric type.')

    def is_valid(self):
        ''' Have all mandatory values set?
        '''
        return self._productNumber != None and self.searchKeywords.is_valid()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import os
import uuid

# import the RestClient class
from epages_client.client import RestClient

# import the script tag create class
from epages_client.dataobjects.script_tag_create import ScriptTagCreate

# import base class for unit testing
from .base_unit_test import BaseUnitTest


class TestScriptTagMethods(BaseUnitTest):
    '''A class for testing script tag related methods on RestClient class'''

    def setUp(self):

        self.client = RestClient(
            os.environ["EPAGES_API_URL"], os.environ["EPAGES_API_TOKEN"])

        self.params = {
            "query": {},
            "param1": "",
            "param2": ""
        }

    def test_001_add_script_tag_no_object(self):

        with self.assertRaises(RuntimeError) as e:
            response = self.client.add_script_tag(self.params)

    def test_002_add_script_tag_invalid_object(self):

        self.params["object"] = None

        with self.assertRaises(TypeError) as e:
            response = self.client.add_script_tag(self.params)

    def test_003_add_script_tag_no_script_url(self):

        script_tag = ScriptTagCreate()

        self.params["object"] = script_tag

        with self.assertRaises(ValueError) as e:
            response = self.client.add_script_tag(self.params)

    def test_004_add_script_tag_false_url(self):

        script_tag = ScriptTagCreate()

        with self.assertRaises(ValueError) as e:
            script_tag.scriptUrl = "foobar"

    def test_005_add_script_tag(self):

        script_tag = ScriptTagCreate()

        script_tag.scriptUrl = "https://code.jquery.com/jquery-3.3.1.min.js"

        self.params["object"] = script_tag

        response = self.client.add_script_tag(self.params)

        self.assertEqual(isinstance(response, dict), True)

    def test_006_get_script_tags(self):

        script_tags = self.client.get_script_tags(self.params)

        self.assertEqual(isinstance(script_tags, dict), True)

    def test_007_delete_script_tag_no_id(self):

        with self.assertRaises(ValueError) as e:
            response = self.client.delete_script_tag(self.params)

    def test_008_delete_script_tag_false_id(self):

        self.params["param1"] = str(uuid.uuid4())

        with self.assertRaises(RuntimeError) as e:
            response = self.client.delete_script_tag(self.params)

    def test_009_delete_script_tag(self):

        script_tags = self.client.get_script_tags()
        script_tag_id = script_tags["items"][0]["scriptTagId"]

        self.params["param1"] = script_tag_id

        response = self.client.delete_script_tag(self.params)

        self.assertEqual(isinstance(response, dict), True)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from six import string_types
from .list_of_objects import ListOfObjects

class ProductSlideshowSequenceUpdate(ListOfObjects):
    """Data object for updating product slideshow sequence"""

    def __init__(self):
        super(ProductSlideshowSequenceUpdate, self).__init__(string_types)

    def get_dict(self):
        return self.get()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

# import the Httphandler Class
from .http_handler import HttpHandler

# import the Product class from dataobjects
from .dataobjects.data_object import DataObject

from pprint import pprint
import json
import os


class RestClient(object):
    '''Client to connect to the ePages REST API'''

    def __init__(self, api_url, token="", verify=True, client_id="", client_secret="", beyond=False):

        # Get an instance of httphandler
        self.http = HttpHandler(api_url, token, verify,
                                client_id, client_secret, beyond)

        # Set currency and locale to be empty by default
        self._currency = ""
        self._locale = ""

        # Get the directory where this file is located
        dir_path = os.path.dirname(os.path.realpath(__file__))

        # Add the method mapping file to the directory path
        method_file = os.path.join(dir_path, "method_mapping.json")

        try:
            # Open the json file containing method mapping
            methods = open(method_file)

            # Load json to the dict
            self.mapping = json.load(methods)

            # Close the file handler
            methods.close()
        except IOError:
            print("Error: method mapping file not found.")
            self.mapping = {}

    def __getattr__(self, attr):
        '''A magic method to catch attributes that don't really exist'''

        # If called attribute is found on the mapping dictionary
        if attr in self.mapping:
            def build_request(params=None):
                '''Called attribute can have params dictionary as a parameter,
                that contains dictionaries for data and query, and variables
                for item id and image name'''

                if params is None:
                    params = {}

                # Get the nested dictionary for called attribute
                method = self.mapping[attr]

                # Check the sent parameters
                method = self.check_params(method, params)

                # Get the needed method function from HttpHandler
                method_func = getattr(self.http, method["method"])

                # Perform the API call
                response = method_func(method["api_dict"])

                return response

            return build_request
        else:
            print("Error: command '%s' not found." % (attr,))

    def check_params(self, method=None, params=None):
        '''
        A function for checking sent parameters; are required ones set
        and are parameters valid.
        '''

        if method is not None and params is not None:

            # If params dictionary doesn't have data and query, add them
            if "data" not in params:
                params["data"] = {}

            if "query" not in params:
                params["query"] = {}

            # If user has set headers
            if "headers" in params and params["headers"]:

                # If headers are found in api_dict, merge headers in params with them
                if "headers" in method["api_dict"] and method["api_dict"]["headers"]:
                    method["api_dict"]["headers"].update(params["headers"])
                else:
                    method["api_dict"]["headers"] = params["headers"]

            # If parameters has a data object
            if "object" in params:

                # Check if the object is an instance of DataObject class
                if not isinstance(params["object"], DataObject):
                    raise TypeError("Invalid object.")

                # Check object validity
                obj = params["object"]
                if not obj.is_valid():
                    raise ValueError("Validate object contents.")

                # If object is valid, get the data in form of dictionary
                # or json patch, and add it to the params
                if method["method"] == "patch":
                    params["data"] = obj.get_patch()
                else:
                    params["data"] = obj.get_dict()

            # If query has not currency or locale, use setted ones
            if "currency" not in params["query"] and self.currency != "":
                params["query"]["currency"] = self.currency

            if "locale" not in params["query"] and self.locale != "":
                params["query"]["locale"] = self.locale

            # Check if there's variable requirements and loop them
            if "require" in method:
                for var in method["require"]:
                    # If requirement has prefix q_, it's a requirement
                    # for query parameter
                    if var.startswith("q_"):
                        # Get the variable value without the prefix
                        missing = var[2:]

                        # If variable is not found in query parameters or it is empty,
                        # raise an error
                        if missing not in params["query"] or params["query"][missing] == "":
                            raise ValueError(
                                "Missing query parameter '" + missing + "'")

                    # Otherwise, it's a required variable
                    else:
                        if var not in params or params[var] == "":
                            raise ValueError("Missing variable '" + var + "'")

            # If there's the first parameter set, add it to the end of the path
            if "param1" in params and params["param1"] != "":
                method["api_dict"]["path"] += "/" + params["param1"]

            if "path2" in method and method["path2"] != "":
                method["api_dict"]["path"] += method["path2"]

            # If there's the second parameter set, add it to the end of the path
            if "param2" in params and params["param2"] != "":
                method["api_dict"]["path"] += "/" + params["param2"]

            # Add data and query dictionaries to the final api_dict
            method["api_dict"]["data"] = params["data"]
            method["api_dict"]["query"] = params["query"]

            return method

    @property
    def currency(self):
        '''Currency getter'''
        return self._currency

    @currency.setter
    def currency(self, value):
        '''Currency setter'''
        self._currency = value

    @property
    def locale(self):
        '''Locale getter'''
        return self._locale

    @locale.setter
    def locale(self, value):
        '''Locale setter'''
        self._locale = value
